@@ -19,6 +19,8 @@
 #include "foreach.hpp"
 #include "joystick.hpp"
 #include "key.hpp"
+#include "level_runner.hpp"
+#include "multiplayer.hpp"
 #include "iphone_controls.hpp"
 
 namespace controls {
@@ -37,10 +39,14 @@ int32_t highest_confirmed[MAX_PLAYERS];
 //for each player, the highest confirmed cycle of ours that they have
 int32_t remote_highest_confirmed[MAX_PLAYERS];
 
+std::map<int, int> our_checksums;
+
 int starting_cycles;
 int nplayers = 1;
 int local_player;
 
+int delay;
+
 int first_invalid_cycle_var = -1;
 
 SDLKey sdlk[NUM_CONTROLS] = {
@@ -49,8 +55,13 @@ SDLKey sdlk[NUM_CONTROLS] = {
 	SDLK_LEFT,
 	SDLK_RIGHT,
 	SDLK_d,
+#if TARGET_OS_IPHONE || TARGET_IPHONE_SIMULATOR
+	SDLK_a,
+	SDLK_b
+#else
 	SDLK_a,
 	SDLK_s
+#endif
 };
 
 int32_t our_highest_confirmed() {
@@ -189,6 +200,31 @@ void get_control_status(int cycle, int player, bool* output)
 	--cycle;
 	cycle -= starting_cycles;
 
+	cycle -= delay;
+	if(cycle < 0) {
+		return;
+	}
+
+	if(player != local_player) {
+		const int breathing_room = highest_confirmed[player] - cycle;
+
+		if(cycle > highest_confirmed[player]) {
+			std::cerr << "DELAYING AND WAITING\n";
+			const int max_delay = 1000;
+			const int end_time = SDL_GetTicks() + max_delay;
+
+			const pause_scope pause;
+			while(cycle > highest_confirmed[player] && SDL_GetTicks() < end_time) {
+				multiplayer::receive();
+			}
+		}
+
+		if(cycle > highest_confirmed[player]) {
+			std::cerr << "ERROR: REMOTE HOST TIMED OUT. GAME ABORTED.\n";
+			throw multiplayer::error();
+		}
+	}
+
 	ASSERT_INDEX_INTO_VECTOR(cycle, controls[player]);
 
 	unsigned char state = controls[player][cycle];
@@ -198,6 +234,11 @@ void get_control_status(int cycle, int player, bool* output)
 	}
 }
 
+void set_delay(int value)
+{
+	delay = value;
+}
+
 void read_control_packet(const char* buf, size_t len)
 {
 	++npackets_received;
@@ -231,6 +272,20 @@ void read_control_packet(const char* buf, size_t len)
 		return;
 	}
 
+	int32_t checksum;
+	memcpy(&checksum, buf, 4);
+	checksum = ntohl(checksum);
+	buf += 4;
+
+	if(checksum && our_checksums[current_cycle-1]) {
+		if(checksum == our_checksums[current_cycle-1]) {
+			std::cerr << "CHECKSUM MATCH FOR " << current_cycle << ": " << checksum << "\n";
+		} else {
+			std::cerr << "CHECKSUM DID NOT MATCH FOR " << current_cycle << ": " << checksum << " VS " << our_checksums[current_cycle-1] << "\n";
+		}
+
+	}
+
 	int32_t highest_cycle;
 	memcpy(&highest_cycle, buf, 4);
 	highest_cycle = ntohl(highest_cycle);
@@ -294,7 +349,6 @@ void read_control_packet(const char* buf, size_t len)
 
 	assert(buf == end_buf);
 
-	fprintf(stderr, "PROCESSED REMOTE PACKET OKAY\n");
 	++ngood_packets;
 }
 
@@ -314,6 +368,12 @@ void write_control_packet(std::vector<char>& v)
 	v.resize(v.size() + 4);
 	memcpy(&v[v.size()-4], &current_cycle_net, 4);
 
+	//write our checksum of game state
+	int32_t checksum = our_checksums[current_cycle-1];
+	int32_t checksum_net = htonl(checksum);
+	v.resize(v.size() + 4);
+	memcpy(&v[v.size()-4], &checksum_net, 4);
+
 	//write our highest confirmed cycle
 	int32_t highest_cycle = htonl(our_highest_confirmed());
 	v.resize(v.size() + 4);
@@ -372,6 +432,11 @@ int last_packet_size()
 	return last_packet_size_;
 }
 
+void set_checksum(int cycle, int sum)
+{
+	our_checksums[cycle] = sum;
+}
+
 void debug_dump_controls()
 {
 	fprintf(stderr, "CONTROLS:");
