@@ -21,6 +21,10 @@ enum CUSTOM_OBJECT_PROPERTY {
 	CUSTOM_OBJECT_X,
 	CUSTOM_OBJECT_Y,
 	CUSTOM_OBJECT_Z,
+	CUSTOM_OBJECT_RELATIVE_X,
+	CUSTOM_OBJECT_RELATIVE_Y,
+	CUSTOM_OBJECT_PARENT,
+	CUSTOM_OBJECT_PIVOT,
 	CUSTOM_OBJECT_ZORDER,
 	CUSTOM_OBJECT_PREVIOUS_Y,
 	CUSTOM_OBJECT_X1,
@@ -45,6 +49,10 @@ enum CUSTOM_OBJECT_PROPERTY {
 	CUSTOM_OBJECT_VELOCITY_Y,
 	CUSTOM_OBJECT_ACCEL_X,
 	CUSTOM_OBJECT_ACCEL_Y,
+	CUSTOM_OBJECT_GRAVITY_SHIFT,
+	CUSTOM_OBJECT_PLATFORM_MOTION_X,
+	CUSTOM_OBJECT_REGISTRY,
+	CUSTOM_OBJECT_GLOBALS,
 	CUSTOM_OBJECT_VARS,
 	CUSTOM_OBJECT_TMP,
 	CUSTOM_OBJECT_GROUP,
@@ -120,6 +128,9 @@ public:
 	entry* get_entry(int slot);
 	const entry* get_entry(int slot) const;
 	int num_slots() const { return NUM_CUSTOM_OBJECT_PROPERTIES; }
+
+private:
+	std::vector<entry> entries_;
 };
 
 #endif
