@@ -1,5 +1,10 @@
+#ifdef TARGET_PANDORA
+#include <GLES/gl.h>
+#include <GLES/glues.h>
+#else
 #include <GL/gl.h>
 #include <GL/glu.h>
+#endif
 #include <SDL.h>
 #include <algorithm>
 #include <cmath>
@@ -15,7 +20,10 @@
 #include "draw_scene.hpp"
 #include "font.hpp"
 #include "foreach.hpp"
+#include "globals.h"
 #include "graphical_font.hpp"
+#include "gui_section.hpp"
+#include "i18n.hpp"
 #include "level.hpp"
 #include "message_dialog.hpp"
 #include "player_info.hpp"
@@ -23,6 +31,7 @@
 #include "raster.hpp"
 #include "speech_dialog.hpp"
 #include "texture.hpp"
+#include "texture_frame_buffer.hpp"
 
 namespace {
 
@@ -31,6 +40,10 @@ std::string& scene_title() {
 	return title;
 }
 
+achievement_ptr current_achievement;
+int current_achievement_duration = 0;
+
+	
 struct screen_flash {
 	graphics::color_transform color, delta;
 	int duration;
@@ -46,6 +59,10 @@ int scene_title_duration_;
 screen_position last_position;
 }
 
+bool is_achievement_displayed() {
+	return current_achievement && current_achievement_duration > 0;
+}
+
 screen_position& last_draw_position()
 {
 	return last_position;
@@ -58,10 +75,17 @@ void screen_color_flash(const graphics::color_transform& color, const graphics::
 }
 
 void set_scene_title(const std::string& msg, int duration) {
-	scene_title() = msg;
+	//explicitly translate all level titles
+	scene_title() = (msg.size() > 0) ? _(msg) : msg;
 	scene_title_duration_ = duration;
 }
 
+void set_displayed_achievement(achievement_ptr a)
+{
+	current_achievement = a;
+	current_achievement_duration = 250;
+}
+
 GLfloat hp_ratio = -1.0;
 void draw_scene(const level& lvl, screen_position& pos, const entity* focus, bool do_draw) {
 	if(focus == NULL && lvl.player()) {
@@ -116,6 +140,8 @@ void draw_scene(const level& lvl, screen_position& pos, const entity* focus, boo
 		const int min_y = lvl.boundaries().y() + graphics::screen_height()/2 - y_screen_pad/2;
 		const int max_y = lvl.boundaries().y2() - graphics::screen_height()/2 + y_screen_pad/2;
 
+		//std::cerr << "BOUNDARIES: " << lvl.boundaries().x() << ", " << lvl.boundaries().x2() << " WIDTH: " << graphics::screen_width() << " PAD: " << x_screen_pad << "\n";
+
 		//we look a certain number of frames ahead -- assuming the focus
 		//keeps moving at the current velocity, we converge toward the point
 		//they will be at in x frames.
@@ -143,6 +169,8 @@ void draw_scene(const level& lvl, screen_position& pos, const entity* focus, boo
 		//find the y point for the camera to converge toward
 		int y = std::min(std::max(focus->feet_y() - graphics::screen_height()/(5*lvl.zoom_level()) + displacement_y*PredictiveFramesVert + vertical_look, min_y), max_y);
 
+		//std::cerr << "POSITION: " << x << "," << y << " IN " << min_x << "," << min_y << "," << max_x << "," << max_y << "\n";
+
 		if(lvl.focus_override().empty() == false) {
 			std::vector<entity_ptr> v = lvl.focus_override();
 			int left = 0, right = 0, top = 0, bottom = 0;
@@ -172,10 +200,13 @@ void draw_scene(const level& lvl, screen_position& pos, const entity* focus, boo
 			y = std::min(std::max((top + bottom)/2 - graphics::screen_height()/(5*lvl.zoom_level()), min_y), max_y);
 		}
 
+
+		//std::cerr << "POSITION2: " << x << "," << y << " IN " << min_x << "," << min_y << "," << max_x << "," << max_y << "\n";
 		if(lvl.lock_screen()) {
 			x = lvl.lock_screen()->x;
 			y = lvl.lock_screen()->y;
 		}
+		//std::cerr << "POSITION3: " << x << "," << y << " IN " << min_x << "," << min_y << "," << max_x << "," << max_y << "\n";
 
 		//for small screens the speech dialog arrows cover the entities they are
 		//pointing to. adjust to that by looking up a little bit.
@@ -303,7 +334,7 @@ void draw_scene(const level& lvl, screen_position& pos, const entity* focus, boo
 
 	debug_console::draw();
 
-	lvl.draw_status();
+	if (!pause_stack) lvl.draw_status();
 
 	if(scene_title_duration_ > 0) {
 		--scene_title_duration_;
@@ -323,6 +354,44 @@ void draw_scene(const level& lvl, screen_position& pos, const entity* focus, boo
 		}
 	}
 	
+	if(current_achievement && current_achievement_duration > 0) {
+		--current_achievement_duration;
+
+		const_gui_section_ptr left = gui_section::get("achievements_left");
+		const_gui_section_ptr right = gui_section::get("achievements_right");
+		const_gui_section_ptr main = gui_section::get("achievements_main");
+
+		const const_graphical_font_ptr title_font = graphical_font::get("white_outline");
+		const const_graphical_font_ptr main_font = graphical_font::get("door_label");
+
+		const std::string title_text = _("Achievement Unlocked!");
+		const std::string name = current_achievement->name();
+		const std::string description = "(" + current_achievement->description() + ")";
+		const int width = std::max<int>(std::max<int>(
+		  title_font->dimensions(title_text).w(),
+		  main_font->dimensions(name).w()),
+		  main_font->dimensions(description).w()
+		  ) + 8;
+		
+		const int xpos = graphics::screen_width() - 16 - left->width() - right->width() - width;
+		const int ypos = 16;
+
+		const GLfloat alpha = current_achievement_duration > 10 ? 1.0 : current_achievement_duration/10.0;
+
+		glColor4f(1.0, 1.0, 1.0, alpha);
+
+		left->blit(xpos, ypos);
+		main->blit(xpos + left->width(), ypos, width, main->height());
+		right->blit(xpos + left->width() + width, ypos);
+
+		title_font->draw(xpos + left->width(), ypos - 10, title_text);
+		main_font->draw(xpos + left->width(), ypos + 24, name);
+
+		glColor4f(0.0, 1.0, 0.0, alpha);
+		main_font->draw(xpos + left->width(), ypos + 48, description);
+		glColor4f(1.0, 1.0, 1.0, 1.0);
+		
+	}
 	
 	if(pos.flip_rotate) {
 		const SDL_Surface* fb = SDL_GetVideoSurface();
