@@ -1,8 +1,14 @@
+#ifdef TARGET_PANDORA
+#include <GLES/gl.h>
+#include <GLES/glues.h>
+#else
 #include <GL/gl.h>
 #include <GL/glu.h>
+#endif
 #include <SDL.h>
 #include <iostream>
 #include <cmath>
+#include <algorithm>
 
 #include <boost/bind.hpp>
 #include <boost/shared_ptr.hpp>
@@ -42,6 +48,7 @@
 #include "preferences.hpp"
 #include "property_editor_dialog.hpp"
 #include "raster.hpp"
+#include "segment_editor_dialog.hpp"
 #include "stats.hpp"
 #include "texture.hpp"
 #include "text_entry_widget.hpp"
@@ -56,6 +63,7 @@
 #include "IMG_savepng.h"
 
 namespace {
+
 //keep a map of editors so that when we edit a level and then later
 //come back to it we'll save all the state we had previously
 std::map<std::string, editor*> all_editors;
@@ -71,6 +79,12 @@ bool g_draw_stats = false;
 void toggle_draw_stats() {
 	g_draw_stats = !g_draw_stats;
 }
+
+bool g_draw_grid = true;
+
+void toggle_draw_grid() {
+	g_draw_grid = !g_draw_grid;
+}
 }
 
 class editor_menu_dialog : public gui::dialog
@@ -115,10 +129,17 @@ class editor_menu_dialog : public gui::dialog
 			"New Object...", "", boost::bind(&editor::new_object_type, &editor_),
 		};
 
+		menu_item duplicate_item = { "Duplicate Object(s)", "ctrl+d", boost::bind(&editor::duplicate_selected_objects, &editor_) };
+
 		std::vector<menu_item> res;
 		foreach(const menu_item& m, items) {
 			res.push_back(m);
 		}
+
+		if(editor_.get_level().editor_selection().empty() == false) {
+			res.push_back(duplicate_item);
+		}
+
 		show_menu(res);
 	}
 
@@ -128,7 +149,8 @@ class editor_menu_dialog : public gui::dialog
 			"Zoom In", "z", boost::bind(&editor::zoom_in, &editor_),
 			editor_.get_level().show_foreground() ? "Hide Foreground" : "Show Foreground", "f", boost::bind(&level::set_show_foreground, &editor_.get_level(), !editor_.get_level().show_foreground()),
 			editor_.get_level().show_background() ? "Hide Background" : "Show Background", "b", boost::bind(&level::set_show_background, &editor_.get_level(), !editor_.get_level().show_background()),
-			g_draw_stats ? "Hide Stats" : "Show Stats", "", toggle_draw_stats
+			g_draw_stats ? "Hide Stats" : "Show Stats", "", toggle_draw_stats,
+			g_draw_grid ? "Hide Grid" : "Show Grid", "", toggle_draw_grid
 		};
 
 		std::vector<menu_item> res;
@@ -201,8 +223,9 @@ class editor_menu_dialog : public gui::dialog
 	gui::widget_ptr context_menu_;
 public:
 	explicit editor_menu_dialog(editor& e)
-	  : gui::dialog(0, 0, graphics::screen_width() - 160, 40), editor_(e)
+	  : gui::dialog(0, 0, preferences::actual_screen_width() - EDITOR_SIDEBAR_WIDTH, EDITOR_MENUBAR_HEIGHT), editor_(e)
 	{
+		set_clear_bg_amount(255);
 		init();
 	}
 
@@ -239,7 +262,7 @@ public:
 		dialog d(0, 0, graphics::screen_width(), graphics::screen_height());
 		d.add_widget(widget_ptr(new label("New Level", graphics::color_white(), 48)));
 		text_entry_widget* entry = new text_entry_widget;
-		d.add_widget(widget_ptr(new label("Name:", graphics::color_white())))
+		d.add_widget(widget_ptr(new label("Filename:", graphics::color_white())))
 		 .add_widget(widget_ptr(entry));
 		d.show_modal();
 		
@@ -335,8 +358,9 @@ class editor_mode_dialog : public gui::dialog
 
 public:
 	explicit editor_mode_dialog(editor& e)
-	  : gui::dialog(graphics::screen_width() - 160, 0, 160, 160), editor_(e)
+	  : gui::dialog(graphics::screen_width() - EDITOR_SIDEBAR_WIDTH, 0, EDITOR_SIDEBAR_WIDTH, 160), editor_(e)
 	{
+		set_clear_bg_amount(255);
 		init();
 	}
 
@@ -349,7 +373,7 @@ public:
 
 		grid_ptr grid(new gui::grid(3));
 		for(int n = 0; n != editor::NUM_TOOLS; ++n) {
-			if(n == editor::TOOL_EDIT_SEGMENTS && editor_.get_level().segment_width() == 0) {
+			if(n == editor::TOOL_EDIT_SEGMENTS && editor_.get_level().segment_width() == 0 && editor_.get_level().segment_height() == 0) {
 				continue;
 			}
 
@@ -453,7 +477,7 @@ rect modify_selected_rect(const editor& e, rect boundaries, int xpos, int ypos)
 	const int y = round_tile_size(ypos);
 
 	if(resizing_left_level_edge) {
-		boundaries = rect::from_coordinates(x, boundaries.y(), boundaries.x2(), boundaries.y2());
+		boundaries = rect(x, boundaries.y(), boundaries.w() + (boundaries.x() - x), boundaries.h());
 		if(e.get_level().segment_width() > 0) {
 			while(boundaries.w()%e.get_level().segment_width() != 0) {
 				boundaries = rect(boundaries.x()-1, boundaries.y(), boundaries.w()+1, boundaries.h());
@@ -462,7 +486,7 @@ rect modify_selected_rect(const editor& e, rect boundaries, int xpos, int ypos)
 	}
 
 	if(resizing_right_level_edge) {
-		boundaries = rect::from_coordinates(boundaries.x(), boundaries.y(), x, boundaries.y2());
+		boundaries = rect(boundaries.x(), boundaries.y(), x - boundaries.x(), boundaries.h());
 		if(e.get_level().segment_width() > 0) {
 			while(boundaries.w()%e.get_level().segment_width() != 0) {
 				boundaries = rect(boundaries.x(), boundaries.y(), boundaries.w()+1, boundaries.h());
@@ -471,11 +495,21 @@ rect modify_selected_rect(const editor& e, rect boundaries, int xpos, int ypos)
 	}
 
 	if(resizing_top_level_edge) {
-		boundaries = rect::from_coordinates(boundaries.x(), y, boundaries.x2(), boundaries.y2());
+		boundaries = rect(boundaries.x(), y, boundaries.w(), boundaries.h() + (boundaries.y() - y));
+		if(e.get_level().segment_height() > 0) {
+			while(boundaries.h()%e.get_level().segment_height() != 0) {
+				boundaries = rect(boundaries.x(), boundaries.y()-1, boundaries.w(), boundaries.h()+1);
+			}
+		}
 	}
 
 	if(resizing_bottom_level_edge) {
-		boundaries = rect::from_coordinates(boundaries.x(), boundaries.y(), boundaries.x2(), y);
+		boundaries = rect(boundaries.x(), boundaries.y(), boundaries.w(), y - boundaries.y());
+		if(e.get_level().segment_height() > 0) {
+			while(boundaries.h()%e.get_level().segment_height() != 0) {
+				boundaries = rect(boundaries.x(), boundaries.y(), boundaries.w(), boundaries.h()+1);
+			}
+		}
 	}
 
 	return boundaries;
@@ -656,6 +690,28 @@ void editor::toggle_facing()
 	}
 }
 
+void editor::duplicate_selected_objects()
+{
+	std::vector<boost::function<void()> > redo, undo;
+	foreach(const entity_ptr& c, lvl_->editor_selection()) {
+		entity_ptr duplicate_obj = c->clone();
+
+		int repeats = 1000;
+		if(!place_entity_in_level_with_large_displacement(*lvl_, *duplicate_obj)) {
+			continue;
+		}
+		
+		redo.push_back(boost::bind(&editor::add_object_to_level, this, duplicate_obj));
+		undo.push_back(boost::bind(&editor::remove_object_from_level, this, duplicate_obj));
+
+
+	}
+
+	execute_command(
+	  boost::bind(execute_functions, redo),
+	  boost::bind(execute_functions, undo));
+}
+
 void editor::process_ghost_objects()
 {
 	lvl_->swap_chars(ghost_objects_);
@@ -697,11 +753,51 @@ unsigned int get_mouse_state(int& mousex, int& mousey) {
 
 	return res;
 }
+
+int editor_resolution_manager_count = 0;
+
+int editor_x_resolution = 0, editor_y_resolution = 0;
+
+struct editor_resolution_manager
+{
+	editor_resolution_manager() :
+	   original_width_(preferences::actual_screen_width()),
+	   original_height_(preferences::actual_screen_height()) {
+		if(!editor_x_resolution) {
+			editor_x_resolution = preferences::actual_screen_width() + EDITOR_SIDEBAR_WIDTH + editor_dialogs::LAYERS_DIALOG_WIDTH;
+			editor_y_resolution = preferences::actual_screen_height() + EDITOR_MENUBAR_HEIGHT;
+		}
+
+		if(++editor_resolution_manager_count == 1) {
+			SDL_Surface* result = graphics::set_video_mode(editor_x_resolution,editor_y_resolution,0,SDL_OPENGL|SDL_RESIZABLE|(preferences::fullscreen() ? SDL_FULLSCREEN : 0));
+
+			if(result) {
+				preferences::set_actual_screen_width(editor_x_resolution);
+				preferences::set_actual_screen_height(editor_y_resolution);
+			} else {
+				editor_x_resolution = preferences::actual_screen_width();
+				editor_y_resolution = preferences::actual_screen_height();
+			}
+		}
+	}
+
+	~editor_resolution_manager() {
+		if(--editor_resolution_manager_count == 0) {
+			preferences::set_actual_screen_width(original_width_);
+			preferences::set_actual_screen_height(original_height_);
+			graphics::set_video_mode(original_width_,original_height_,0,SDL_OPENGL|(preferences::resizable() ? SDL_RESIZABLE : 0)|(preferences::fullscreen() ? SDL_FULLSCREEN : 0));
+		}
+	}
+
+	int original_width_, original_height_;
+};
+
 }
 
 void editor::edit_level()
 {
-	reset_dialog_positions();
+	editor_resolution_manager resolution_manager;
+
 	stats::flush();
 	try {
 		load_stats();
@@ -752,6 +848,7 @@ void editor::edit_level()
 
 		int mousex, mousey;
 		const unsigned int buttons = get_mouse_state(mousex, mousey);
+		const Uint8* keystate = SDL_GetKeyState(NULL); 
 
 		if(buttons == 0) {
 			drawing_rect_ = false;
@@ -768,6 +865,7 @@ void editor::edit_level()
 		prev_mousex = mousex;
 		prev_mousey = mousey;
 
+		
 		const int selectx = round_tile_size(xpos_ + mousex*zoom_);
 		const int selecty = round_tile_size(ypos_ + mousey*zoom_);
 
@@ -799,7 +897,7 @@ void editor::edit_level()
 			//remove ghost objects and re-add them. This guarantees ghost
 			//objects always remain at the end of the level ordering.
 			remove_ghost_objects();
-			entity_ptr c = lvl_->get_character_at_point(xpos_ + mousex*zoom_, ypos_ + mousey*zoom_);
+			entity_ptr c = lvl_->get_next_character_at_point(xpos_ + mousex*zoom_, ypos_ + mousey*zoom_);
 			foreach(const entity_ptr& ghost, ghost_objects_) {
 				lvl_->add_character(ghost);
 			}
@@ -826,10 +924,14 @@ void editor::edit_level()
 				remove_ghost_objects();
 				ghost_objects_.clear();
 			}
-		} else if(object_mode && lvl_->editor_highlight()) {
+		}else if(object_mode && lvl_->editor_highlight()) {
 			//we're handling objects, and a button is down, and we have an
 			//object under the mouse. This means we are dragging something.
-			handle_object_dragging(mousex, mousey);
+
+			// check if cursor is not in the sidebar!
+			if (mousex < editor_mode_dialog_->x()) {
+				handle_object_dragging(mousex, mousey);
+			}
 		} else if(drawing_rect_) {
 			handle_drawing_rect(mousex, mousey);
 		}
@@ -838,7 +940,7 @@ void editor::edit_level()
 			//not in object mode, the picker still highlights objects,
 			//though it won't create ghosts, so remove all ghosts.
 			if(tool() == TOOL_PICKER) {
-				entity_ptr c = lvl_->get_character_at_point(xpos_ + mousex*zoom_, ypos_ + mousey*zoom_);
+				entity_ptr c = lvl_->get_next_character_at_point(xpos_ + mousex*zoom_, ypos_ + mousey*zoom_);
 				lvl_->set_editor_highlight(c);
 			} else {
 				lvl_->set_editor_highlight(entity_ptr());
@@ -908,11 +1010,16 @@ void editor::edit_level()
 			case SDL_VIDEORESIZE: {
 				const SDL_ResizeEvent* const resize = reinterpret_cast<SDL_ResizeEvent*>(&event);
 
-				preferences::set_actual_screen_width(resize->w);
-				preferences::set_actual_screen_height(resize->h);
-				SDL_SetVideoMode(resize->w,resize->h,0,SDL_OPENGL|SDL_RESIZABLE|(preferences::fullscreen() ? SDL_FULLSCREEN : 0));
+				SDL_Surface* result = graphics::set_video_mode(resize->w,resize->h,0,SDL_OPENGL|SDL_RESIZABLE|(preferences::fullscreen() ? SDL_FULLSCREEN : 0));
 				
-				reset_dialog_positions();
+				if(result) {
+					editor_x_resolution = resize->w;
+					editor_y_resolution = resize->h;
+					preferences::set_actual_screen_width(resize->w);
+					preferences::set_actual_screen_height(resize->h);
+				
+					reset_dialog_positions();
+				}
 
 				continue;
 			}
@@ -931,23 +1038,49 @@ void editor::edit_level()
 
 void editor::reset_dialog_positions()
 {
+	if(editor_mode_dialog_) {
+		editor_mode_dialog_->set_loc(graphics::screen_width() - editor_mode_dialog_->width(), editor_mode_dialog_->y());
+	}
 
-#define SET_DIALOG_POS(d) if(d) { d->set_loc(graphics::screen_width() - d->width(), d->y()); }
-				SET_DIALOG_POS(editor_mode_dialog_);
+#define SET_DIALOG_POS(d) if(d) { \
+	d->set_loc(graphics::screen_width() - d->width(), d->y()); \
+	\
+	d->set_dim(d->width(), \
+	 std::max<int>(10, preferences::actual_screen_height() - d->y())); \
+}
 				SET_DIALOG_POS(character_dialog_);
-				SET_DIALOG_POS(layers_dialog_);
 				SET_DIALOG_POS(group_property_dialog_);
 				SET_DIALOG_POS(property_dialog_);
 				SET_DIALOG_POS(tileset_dialog_);
 #undef SET_DIALOG_POS
+
+				if(layers_dialog_ && editor_mode_dialog_) {
+					layers_dialog_->set_loc(editor_mode_dialog_->x() - layers_dialog_->width(), EDITOR_MENUBAR_HEIGHT);
+					layers_dialog_->set_dim(layers_dialog_->width(), preferences::actual_screen_height() - EDITOR_MENUBAR_HEIGHT);
+				}
+
+				if(editor_menu_dialog_ && editor_mode_dialog_) {
+					editor_menu_dialog_->set_dim(preferences::actual_screen_width() - editor_mode_dialog_->width(), editor_menu_dialog_->height());
+				}
 }
 
+namespace {
+	bool sort_entity_zsub_orders(const entity_ptr& a, const entity_ptr& b) {
+	return a->zsub_order() < b->zsub_order();
+}
+}
+
+
 void editor::handle_key_press(const SDL_KeyboardEvent& key)
 {
 	if(key.keysym.sym == SDLK_s && (key.keysym.mod&KMOD_ALT)) {
 		IMG_SaveFrameBuffer((std::string(preferences::user_data_path()) + "screenshot.png").c_str(), 5);
 	}
 
+	if(key.keysym.sym == SDLK_d && key.keysym.mod&KMOD_CTRL) {
+		duplicate_selected_objects();
+	}
+
 	if(key.keysym.sym == SDLK_u) {
 		undo_command();
 	}
@@ -960,7 +1093,63 @@ void editor::handle_key_press(const SDL_KeyboardEvent& key)
 	if(key.keysym.sym == SDLK_z) {
 		zoom_in();
 	}
+	
+	if(key.keysym.sym == SDLK_KP8) {
+		foreach(const entity_ptr& e, lvl_->editor_selection()){
+			execute_command(boost::bind(&editor::move_object, this, e, e->x(),e->y()-2),
+							boost::bind(&editor::move_object,this,e,e->x(),e->y()));
+		}
+	}
 
+	if(key.keysym.sym == SDLK_KP5) {
+		foreach(const entity_ptr& e, lvl_->editor_selection()){
+			execute_command(boost::bind(&editor::move_object, this, e, e->x(),e->y()+2),
+							boost::bind(&editor::move_object,this,e,e->x(),e->y()));
+		}
+	}
+	
+	if(key.keysym.sym == SDLK_KP4) {
+		foreach(const entity_ptr& e, lvl_->editor_selection()){
+			execute_command(boost::bind(&editor::move_object, this, e, e->x()-2,e->y()),
+							boost::bind(&editor::move_object,this,e,e->x(),e->y()));
+		}
+	}
+	
+	if(key.keysym.sym == SDLK_KP6) {
+		foreach(const entity_ptr& e, lvl_->editor_selection()){
+			execute_command(boost::bind(&editor::move_object, this, e, e->x()+2,e->y()),
+							boost::bind(&editor::move_object,this,e,e->x(),e->y()));
+		}
+	}
+	
+	if(key.keysym.sym == SDLK_EQUALS || key.keysym.sym == SDLK_MINUS ) {
+		if(lvl_->editor_selection().size() > 1){
+			
+			//store them in a new container
+			std::vector <entity_ptr> v2;
+			foreach(const entity_ptr& e, lvl_->editor_selection()){
+				v2.push_back(e.get());
+			}
+			//sort this container in ascending zsub_order
+			std::sort(v2.begin(),v2.end(), sort_entity_zsub_orders);
+					
+			//if it was +, then move the backmost object in front of the frontmost object.
+			//if it was -, do vice versa (frontmost object goes behind backmost objet)
+			if(key.keysym.sym == SDLK_EQUALS){
+				execute_command(boost::bind(&entity::set_zsub_order, v2.front(), v2.back()->zsub_order()+1),
+								boost::bind(&entity::set_zsub_order, v2.front(), v2.front()->zsub_order() ));
+				
+				//v2.front()->set_zsub_order( v2.back()->zsub_order() +1);
+			}else if(key.keysym.sym == SDLK_MINUS){
+				execute_command(boost::bind(&entity::set_zsub_order, v2.back(), v2.front()->zsub_order()-1),
+								boost::bind(&entity::set_zsub_order, v2.back(), v2.back()->zsub_order() ));
+				
+				//v2.back()->set_zsub_order( v2.front()->zsub_order() -1);
+			}
+		}
+	}
+	
+	
 	if(key.keysym.sym == SDLK_x) {
 		zoom_out();
 	}
@@ -1056,7 +1245,7 @@ void editor::handle_key_press(const SDL_KeyboardEvent& key)
 void editor::handle_scrolling()
 {
 	const int ScrollSpeed = 24*zoom_;
-	const int FastScrollSpeed = 48*zoom_;
+	const int FastScrollSpeed = 384*zoom_;
 
 	if(key_[SDLK_LEFT]) {
 		xpos_ -= ScrollSpeed;
@@ -1214,10 +1403,12 @@ void editor::handle_mouse_button_down(const SDL_MouseButtonEvent& event)
 	if(tool() == TOOL_EDIT_SEGMENTS) {
 		if(point_in_rect(point(anchorx_, anchory_), lvl_->boundaries())) {
 			const int xpos = anchorx_ - lvl_->boundaries().x();
-			const int segment = xpos/lvl_->segment_width();
+			const int ypos = anchory_ - lvl_->boundaries().y();
+			const int segment = lvl_->segment_width() ? xpos/lvl_->segment_width() : ypos/lvl_->segment_height();
 
 			if(selected_segment_ == -1) {
 				selected_segment_ = segment;
+				segment_dialog_->set_segment(segment);
 			} else if(buttons&SDL_BUTTON_RIGHT) {
 				if(segment != selected_segment_ && selected_segment_ >= 0) {
 					variant next = lvl_->get_var(formatter() << "segments_after_" << selected_segment_);
@@ -1240,6 +1431,7 @@ void editor::handle_mouse_button_down(const SDL_MouseButtonEvent& event)
 			}
 		} else {
 			selected_segment_ = -1;
+			segment_dialog_->set_segment(selected_segment_);
 		}
 	} else if(tool() == TOOL_PICKER) {
 		if(lvl_->editor_highlight()) {
@@ -1379,9 +1571,15 @@ void editor::handle_mouse_button_down(const SDL_MouseButtonEvent& event)
 			//clicked in an illegal position to place an object.
 
 		} else if(c->is_human() && lvl_->player()) {
-			execute_command(
-			  boost::bind(&editor::add_object_to_level, this, c),
-			  boost::bind(&editor::add_object_to_level, this, &lvl_->player()->get_entity()));
+			if(!shift_pressed) {
+				execute_command(
+				  boost::bind(&editor::add_object_to_level, this, c),
+				  boost::bind(&editor::add_object_to_level, this, &lvl_->player()->get_entity()));
+			} else {
+				execute_command(
+				  boost::bind(&editor::add_multi_object_to_level, this, c),
+				  boost::bind(&editor::add_object_to_level, this, &lvl_->player()->get_entity()));
+			}
 
 		} else {
 			execute_command(
@@ -1697,6 +1895,11 @@ void editor::add_tile_rect(int x1, int y1, int x2, int y2)
 
 void editor::remove_tile_rect(int x1, int y1, int x2, int y2)
 {
+	x1 += ((100 - tilesets[cur_tileset_].x_speed)*xpos_)/100;
+	x2 += ((100 - tilesets[cur_tileset_].x_speed)*xpos_)/100;
+	y1 += ((100 - tilesets[cur_tileset_].y_speed)*ypos_)/100;
+	y2 += ((100 - tilesets[cur_tileset_].y_speed)*ypos_)/100;
+
 	if(x2 < x1) {
 		std::swap(x1, x2);
 	}
@@ -1892,11 +2095,23 @@ void editor::change_tool(EDIT_TOOL tool)
 		current_dialog_ = property_dialog_.get();
 		break;
 	}
+	case TOOL_EDIT_SEGMENTS: {
+
+		if(!segment_dialog_) {
+			segment_dialog_.reset(new editor_dialogs::segment_editor_dialog(*this));
+		}
+	
+		current_dialog_ = segment_dialog_.get();
+		segment_dialog_->set_segment(selected_segment_);
+		break;
+	}
 	}
 
 	if(editor_mode_dialog_) {
 		editor_mode_dialog_->init();
 	}
+
+	reset_dialog_positions();
 }
 
 void editor::save_level_as(const std::string& fname)
@@ -2149,16 +2364,37 @@ void editor::draw() const
 
 	glPopMatrix();
 
-	//draw grid
-	glDisable(GL_TEXTURE_2D);
-	glDisableClientState(GL_TEXTURE_COORD_ARRAY);
-	varray.clear();
-	glColor4ub(255, 255, 255, 64);
-	for(int x = -TileSize - (xpos_/zoom_)%TileSize; x < graphics::screen_width(); x += 32/zoom_) {
-		varray.push_back(x); varray.push_back(0);
-		varray.push_back(x); varray.push_back(graphics::screen_height());
+	//draw the difficulties of segments.
+	if(lvl_->segment_width() > 0 || lvl_->segment_height() > 0) {
+		const int seg_width = lvl_->segment_width() ? lvl_->segment_width() : lvl_->boundaries().w();
+		const int seg_height = lvl_->segment_height() ? lvl_->segment_height() : lvl_->boundaries().h();
+		rect boundaries = modify_selected_rect(*this, lvl_->boundaries(), selectx, selecty);
+		int seg = 0;
+		for(int ypos = boundaries.y(); ypos < boundaries.y2(); ypos += seg_height) {
+			const int y1 = ypos/zoom_;
+			for(int xpos = boundaries.x(); xpos < boundaries.x2(); xpos += seg_width) {
+				const int difficulty = lvl_->get_var(formatter() << "segment_difficulty_start_" << seg).as_int();
+//				if(difficulty) {
+					graphics::blit_texture(font::render_text(formatter() << "Difficulty: " << difficulty, graphics::color_white(), 14), (xpos - xpos_)/zoom_, y1 - 20 - ypos_/zoom_);
+//				}
+			
+				++seg;
+			}
+		}
 	}
 
+	//draw grid
+	if(g_draw_grid){
+	   glDisable(GL_TEXTURE_2D);
+	   glDisableClientState(GL_TEXTURE_COORD_ARRAY);
+	   varray.clear();
+	   glColor4ub(255, 255, 255, 64);
+	   for(int x = -TileSize - (xpos_/zoom_)%TileSize; x < graphics::screen_width(); x += 32/zoom_) {
+		   varray.push_back(x); varray.push_back(0);
+		   varray.push_back(x); varray.push_back(graphics::screen_height());
+	   }
+	}   
+
 	for(int y = -TileSize - (ypos_/zoom_)%TileSize; y < graphics::screen_height(); y += 32/zoom_) {
 		varray.push_back(0); varray.push_back(y);
 		varray.push_back(graphics::screen_width()); varray.push_back(y);
@@ -2234,6 +2470,17 @@ void editor::draw() const
 				normal_color.add_to_vector(&carray);
 			}
 		}
+
+		if(lvl_->segment_height() > 0) {
+			for(int ypos = boundaries.y() + lvl_->segment_height(); ypos < boundaries.y2(); ypos += lvl_->segment_height()) {
+				varray.push_back(x1 - xpos_/zoom_);
+				varray.push_back((ypos - ypos_)/zoom_);
+				varray.push_back(x2 - xpos_/zoom_);
+				varray.push_back((ypos - ypos_)/zoom_);
+				normal_color.add_to_vector(&carray);
+				normal_color.add_to_vector(&carray);
+			}
+		}
 		
 		glEnableClientState(GL_COLOR_ARRAY);
 		glVertexPointer(2, GL_FLOAT, 0, &varray.front());
@@ -2255,7 +2502,9 @@ void editor::draw() const
 	}
 
 	if(tool() == TOOL_EDIT_SEGMENTS && selected_segment_ >= 0) {
-		rect area = rect(lvl_->boundaries().x() + selected_segment_*lvl_->segment_width(), lvl_->boundaries().y(), lvl_->segment_width(), lvl_->boundaries().h());
+		rect area = rect(lvl_->boundaries().x() + selected_segment_*lvl_->segment_width(), lvl_->boundaries().y() + selected_segment_*lvl_->segment_height(),
+		lvl_->segment_width() ? lvl_->segment_width() : lvl_->boundaries().w(),
+		lvl_->segment_height() ? lvl_->segment_height() : lvl_->boundaries().h());
 		area = rect((area.x() - xpos_)/zoom_, (area.y() - ypos_)/zoom_,
 		            area.w()/zoom_, area.h()/zoom_);
 		graphics::draw_rect(area, graphics::color(255, 255, 0, 64));
@@ -2264,7 +2513,9 @@ void editor::draw() const
 		if(next.is_list()) {
 			for(int n = 0; n != next.num_elements(); ++n) {
 				const int segment = next[n].as_int();
-				rect area = rect(lvl_->boundaries().x() + segment*lvl_->segment_width(), lvl_->boundaries().y(), lvl_->segment_width(), lvl_->boundaries().h());
+				rect area = rect(lvl_->boundaries().x() + segment*lvl_->segment_width(), lvl_->boundaries().y() + segment*lvl_->segment_height(),
+				lvl_->segment_width() ? lvl_->segment_width() : lvl_->boundaries().w(),
+				lvl_->segment_height() ? lvl_->segment_height() : lvl_->boundaries().h());
 				area = rect((area.x() - xpos_)/zoom_, (area.y() - ypos_)/zoom_,
 				            area.w()/zoom_, area.h()/zoom_);
 				graphics::draw_rect(area, graphics::color(255, 0, 0, 64));
@@ -2312,8 +2563,10 @@ void editor::draw_selection(int xoffset, int yoffset) const
 	const uint16_t stipple_mask = (stipple_bits&0xFFFF) | ((stipple_bits&0xFFFF0000) >> 16);
 	
 	glColor4ub(255, 255, 255, 255);
+#ifndef SDL_VIDEO_OPENGL_ES
 	glEnable(GL_LINE_STIPPLE);
 	glLineStipple(1, stipple_mask);
+#endif
 	std::vector<GLfloat>& varray = graphics::global_vertex_array();
 	varray.clear();
 	foreach(const point& p, tile_selection_.tiles) {
@@ -2343,8 +2596,10 @@ void editor::draw_selection(int xoffset, int yoffset) const
 	}
 	glVertexPointer(2, GL_FLOAT, 0, &varray.front());
 	glDrawArrays(GL_LINES, 0, varray.size()/2);
+#ifndef SDL_VIDEO_OPENGL_ES
 	glDisable(GL_LINE_STIPPLE);
 	glLineStipple(1, 0xFFFF);
+#endif
 }
 
 void editor::run_script(const std::string& id)
@@ -2469,6 +2724,12 @@ void editor::edit_level_properties()
 	prop_dialog.show_modal();
 }
 
+void editor::add_multi_object_to_level(entity_ptr e)
+{
+	lvl_->add_multi_player(e);
+	e->handle_event("editor_added");
+}
+
 void editor::add_object_to_level(entity_ptr e)
 {
 	lvl_->add_character(e);
