@@ -58,10 +58,12 @@ public:
 
 	void set_centi_x(int x) { x_ = x; calculate_solid_rect(); }
 	void set_centi_y(int y) { y_ = y; calculate_solid_rect(); }
+	virtual void set_zsub_order(const int zsub_order) = 0;
 
 	int x() const { return x_/100 - (x_ < 0 && x_%100 ? 1 : 0); }
 	int y() const { return y_/100 - (y_ < 0 && y_%100 ? 1 : 0); }
 	virtual int zorder() const { return 0; }
+	virtual int zsub_order() const { return 0; }
 
 	virtual const std::pair<int,int>* position_scale_millis() const { return 0; }
 
@@ -106,6 +108,9 @@ public:
 	int last_move_x() const;
 	int last_move_y() const;
 
+	void set_platform_motion_x(int value);
+	int platform_motion_x() const;
+
 	bool face_right() const { return face_right_; }
 	virtual void set_face_right(bool facing);
 
@@ -120,7 +125,9 @@ public:
 
 	virtual int teleport_offset_x() const { return 0; }
 	virtual int teleport_offset_y() const { return 0; }
+	virtual bool no_move_to_standing() const { return 0; }
 
+	virtual void die_with_no_event() = 0;
 	virtual bool is_active(const rect& screen_area) const = 0;
 	virtual bool dies_on_inactive() const { return false; } 
 	virtual bool always_active() const { return false; } 
@@ -232,6 +239,10 @@ public:
 	virtual const std::vector<light_ptr>& lights() const = 0;
 	virtual void swap_lights(std::vector<light_ptr>& lights) = 0;
 
+	point pivot(const std::string& name) const;
+
+	virtual bool appears_at_difficulty(int difficulty) const = 0;
+
 protected:
 
 	virtual const solid_info* calculate_solid() const = 0;
@@ -254,6 +265,7 @@ protected:
 
 	const std::vector<entity_ptr>& attached_objects() const { return attached_objects_; }
 
+
 private:
 	virtual void control(const level& lvl) = 0;
 
@@ -294,6 +306,8 @@ private:
 	rect solid_rect_, frame_rect_, platform_rect_;
 	const solid_info* solid_;
 	const solid_info* platform_;
+
+	int platform_motion_x_;
 };
 
 #endif
