@@ -4,6 +4,7 @@
 #include "controls.hpp"
 #include "foreach.hpp"
 #include "formula_constants.hpp"
+#include "i18n.hpp"
 #include "key_button.hpp"
 #include "preferences.hpp"
 #include "raster.hpp"
@@ -53,6 +54,8 @@ variant get_constant(const std::string& id)
 		return variant(gui::get_key_name(controls::get_sdlkey(controls::CONTROL_TONGUE)));
 	} else if(id == "ATTACK_KEY") {
 		return variant(gui::get_key_name(controls::get_sdlkey(controls::CONTROL_ATTACK)));
+	} else if(id == "LOCALE") {
+		return variant(i18n::get_locale());
 	}
 
 	if(constants_stack.empty() == false) {
@@ -103,7 +106,7 @@ constants_loader::constants_loader(wml::const_node_ptr node) : same_as_base_(fal
 		std::cerr << "\n";
 	}
 
-	std::cerr << "ADD CONSTANTS_STACK ";
+	//std::cerr << "ADD CONSTANTS_STACK ";
 	for(constants_map::const_iterator i = m.begin(); i != m.end(); ++i) {
 		std::cerr << i->first << " ";
 	}
@@ -117,7 +120,7 @@ constants_loader::~constants_loader()
 {
 	ASSERT_EQ(constants_stack.empty(), false);
 	constants_stack.pop_back();
-	std::cerr << "REMOVE CONSTANTS_STACK\n";
+	//std::cerr << "REMOVE CONSTANTS_STACK\n";
 }
 
 }
