@@ -20,6 +20,7 @@
 #include "formula_callable.hpp"
 #include "formula_callable_utils.hpp"
 #include "formula_function.hpp"
+#include "unit_test.hpp"
 
 #include "SDL.h"
 
@@ -371,6 +372,134 @@ private:
 	}
 };
 
+class sin_function : public function_expression {	//Interprets two least significant digits as after decimal.
+public:
+	explicit sin_function(const args_list& args)
+	     : function_expression("sin", args, 1, 1)
+	{}
+
+private:
+	variant execute(const formula_callable& variables) const {
+		const int value = args()[0]->evaluate(variables).as_int();
+		const double angle = (static_cast<double>(value));
+		//std::cerr << "got: " << value << "\n" << "ret: " << sin(angle/57.3)*36 << "\n";
+		return variant(static_cast<int>(sin(angle/57.3)*100));
+	}
+};
+
+class cos_function : public function_expression {	//Interprets two least significant digits as after decimal.
+public:
+	explicit cos_function(const args_list& args)
+	     : function_expression("cos", args, 1, 1)
+	{}
+
+private:
+	variant execute(const formula_callable& variables) const {
+		const int value = args()[0]->evaluate(variables).as_int();
+		const double angle = (static_cast<double>(value));
+		//std::cerr << "got: " << value << "\n" << "ret: " << cos(angle/57.3)*36 << "\n";
+		return variant(static_cast<int>(cos(angle/57.3)*100));
+	}
+};
+
+class tan_function : public function_expression {	//Interprets two least significant digits as after decimal.
+public:
+	explicit tan_function(const args_list& args)
+	     : function_expression("tan", args, 1, 1)
+	{}
+
+private:
+	variant execute(const formula_callable& variables) const {
+		const int value = args()[0]->evaluate(variables).as_int();
+		const double angle = (static_cast<double>(value));
+		//std::cerr << "got: " << value << "\n" << "ret: " << tan(angle/57.3)*36 << "\n";
+		return variant(static_cast<int>(tan(angle/57.3)*100));
+	}
+};
+
+class asin_function : public function_expression {	//Interprets two least significant digits as after decimal.
+public:
+	explicit asin_function(const args_list& args)
+	     : function_expression("asin", args, 1, 1)
+	{}
+
+private:
+	variant execute(const formula_callable& variables) const {
+		const int value = args()[0]->evaluate(variables).as_int();
+		const double angle = (static_cast<double>(value)/100.0);
+		//std::cerr << "got: " << angle << "\n" << "ret: " << asin(angle)*100 << "\n";
+		return variant(static_cast<int>(asin(angle)*57.29577951308232087));
+	}
+};
+
+class acos_function : public function_expression {	//Interprets two least significant digits as after decimal.
+public:
+	explicit acos_function(const args_list& args)
+	     : function_expression("acos", args, 1, 1)
+	{}
+
+private:
+	variant execute(const formula_callable& variables) const {
+		const int value = args()[0]->evaluate(variables).as_int();
+		const double angle = (static_cast<double>(value)/100.0);
+		return variant(static_cast<int>(acos(angle)*57.29577951308232087));
+	}
+};
+
+class sqrt_function : public function_expression {
+public:
+	explicit sqrt_function(const args_list& args)
+	     : function_expression("sqrt", args, 1, 1)
+	{}
+
+private:
+	variant execute(const formula_callable& variables) const {
+		const int value = args()[0]->evaluate(variables).as_int();
+		return variant(static_cast<int>(sqrt(value)));
+	}
+};
+
+class angle_function : public function_expression {
+public:
+	explicit angle_function(const args_list& args)
+	     : function_expression("angle", args, 4, 4)
+	{}
+
+private:
+	variant execute(const formula_callable& variables) const {
+		const float a = args()[0]->evaluate(variables).as_int();
+		const float b = args()[1]->evaluate(variables).as_int();
+		const float c = args()[2]->evaluate(variables).as_int();
+		const float d = args()[3]->evaluate(variables).as_int();
+		return variant(static_cast<int>(round((atan2(a-c, b-d)*57.29577951308232087+90)*VARIANT_DECIMAL_PRECISION)*-1), variant::DECIMAL_VARIANT);
+	}
+};
+
+class orbit_function : public function_expression {	//Takes x1, y1, distance from, angle from, returns x2, y2.
+public:
+	explicit orbit_function(const args_list& args)
+	     : function_expression("orbit", args, 4,4)
+	{}
+
+private:
+	variant execute(const formula_callable& variables) const {
+		const float x = args()[0]->evaluate(variables).as_int();
+		const float y = args()[1]->evaluate(variables).as_int();
+		const float ang = args()[2]->evaluate(variables).as_int();
+		const float dist = args()[3]->evaluate(variables).as_int();
+		
+		const float u = (dist * cos((ang/57.29577951308232087)/1000)) + x;
+		const float v = (dist * sin((ang/57.29577951308232087)/1000)) + y;
+
+		std::vector<variant> result;
+		result.reserve(2);
+		result.push_back(variant(static_cast<int>(u*VARIANT_DECIMAL_PRECISION), variant::DECIMAL_VARIANT));
+		result.push_back(variant(static_cast<int>(v*VARIANT_DECIMAL_PRECISION), variant::DECIMAL_VARIANT));
+		
+		return variant(&result);
+	}
+};
+
 namespace {
 class variant_comparator : public formula_callable {
 	expression_ptr expr_;
@@ -426,6 +555,39 @@ private:
 	}
 };
 
+namespace {
+	void flatten_items( variant items, std::vector<variant>* output){
+		for(size_t n = 0; n != items.num_elements(); ++n) {
+			
+			if( items[n].is_list() ){
+				flatten_items(items[n], output);
+			} else {
+				output->push_back(items[n]);
+			}
+			
+		}
+	}
+	
+}
+	
+class flatten_function : public function_expression {
+public:
+	explicit flatten_function(const args_list& args)
+	: function_expression("flatten", args, 1, 1)
+	{}
+	
+private:
+	variant execute(const formula_callable& variables) const {
+		variant input = args()[0]->evaluate(variables);
+		std::vector<variant> output;
+
+		flatten_items(input, &output);
+		
+		return variant(&output);
+	}
+};
+	
+	
 class filter_function : public function_expression {
 public:
 	explicit filter_function(const args_list& args)
@@ -502,6 +664,8 @@ private:
 		} else {
 			map_formula_callable* self_callable = new map_formula_callable;
 			formula_callable_ptr callable(self_callable);
+			self_callable->add("context", variant(&variables));
+
 			const std::string self = args()[1]->evaluate(variables).as_string();
 			for(size_t n = 0; n != items.num_elements(); ++n) {
 				self_callable->add(self, items[n]);
@@ -596,17 +760,24 @@ private:
 			}
 		} else {
 			static const std::string index_str = "index";
+			static const std::string context_str = "context";
 			map_formula_callable* self_callable = new map_formula_callable;
 			formula_callable_ptr callable_ref(self_callable);
-			self_callable->add("context", variant(&variables));
+			self_callable->add(context_str, variant(&variables));
 			const std::string self = args()[1]->evaluate(variables).as_string();
-			for(size_t n = 0; n != items.num_elements(); ++n) {
-				self_callable->add(self, items[n]);
-				self_callable->add(index_str, variant(n));
 
-				formula_callable_ptr callable_backup(new formula_callable_with_backup(*self_callable, variables));
-				const variant val = args().back()->evaluate(*callable_backup);
-				vars.push_back(val);
+			variant& self_variant = self_callable->add_direct_access(self);
+
+			//the variant representing the index we are currently at.
+			variant& index_variant = self_callable->add_direct_access(index_str);
+			index_variant = variant(0);
+
+			formula_callable_ptr callable_backup(new formula_callable_with_backup(*self_callable, variables));
+
+			const int nelements = items.num_elements();
+			for(int& n = index_variant.int_addr(); n != nelements; ++n) {
+				self_variant = items[n];
+				vars.push_back(args().back()->evaluate(*callable_backup));
 			}
 		}
 
@@ -637,21 +808,35 @@ private:
 class range_function : public function_expression {
 public:
 	explicit range_function(const args_list& args)
-	  : function_expression("range", args, 1, 1)
+	  : function_expression("range", args, 1, 2)
 	{}
 private:
 	variant execute(const formula_callable& variables) const {
-		const int nelem = args()[0]->evaluate(variables).as_int();
+		int start = args().size() > 1 ? args()[0]->evaluate(variables).as_int() : 0;
+		int end = args()[args().size() > 1 ? 1 : 0]->evaluate(variables).as_int();
+		bool reverse = false;
+		if(end < start) {
+			std::swap(start, end);
+			++start;
+			++end;
+			reverse = true;
+		}
+		const int nelem = end - start;
+
 		std::vector<variant> v;
 
 		if(nelem > 0) {
 			v.reserve(nelem);
 
 			for(int n = 0; n < nelem; ++n) {
-				v.push_back(variant(n));
+				v.push_back(variant(start+n));
 			}
 		}
 
+		if(reverse) {
+			std::reverse(v.begin(), v.end());
+		}
+
 		return variant(&v);
 	}
 };
@@ -691,8 +876,8 @@ private:
 		if(list.num_elements() == 0) {
 			return variant();
 		}
-		int begin_index = args()[1]->evaluate(variables).as_int()%list.num_elements();
-		int end_index = args()[2]->evaluate(variables).as_int()%list.num_elements();
+		int begin_index = args()[1]->evaluate(variables).as_int()%(list.num_elements()+1);
+		int end_index = args()[2]->evaluate(variables).as_int()%(list.num_elements()+1);
 		if(end_index >= begin_index) {
 			std::vector<variant> result;
 			result.reserve(end_index - begin_index);
@@ -917,7 +1102,16 @@ functions_map& get_functions_map() {
 		FUNCTION(max);
 		FUNCTION(choose);
 		FUNCTION(wave);
+		FUNCTION(sin);
+		FUNCTION(cos);
+		FUNCTION(tan);
+		FUNCTION(asin);
+		FUNCTION(acos);
+		FUNCTION(sqrt);
+		FUNCTION(angle);
+		FUNCTION(orbit);
 		FUNCTION(sort);
+		FUNCTION(flatten);
 		FUNCTION(filter);
 		FUNCTION(mapping);
 		FUNCTION(find);
@@ -1013,3 +1207,38 @@ function_expression::function_expression(
 
 
 }
+
+UNIT_TEST(flatten_function) {
+	CHECK(game_logic::formula("flatten([1,[2,3]])").execute() == game_logic::formula("[1,2,3]").execute(), "test failed");
+	CHECK(game_logic::formula("flatten([1,2,3,[[4,5],6]])").execute() == game_logic::formula("[1,2,3,4,5,6]").execute(), "test failed");
+	CHECK(game_logic::formula("flatten([[1,2,3,4],5,6])").execute() == game_logic::formula("[1,2,3,4,5,6]").execute(), "test failed");
+	CHECK(game_logic::formula("flatten([[[0,2,4],6,8],10,[12,14]])").execute() == game_logic::formula("[0,2,4,6,8,10,12,14]").execute(), "test failed");
+}
+
+
+BENCHMARK(map_function) {
+	using namespace game_logic;
+
+	static map_formula_callable* callable = NULL;
+	static variant callable_var;
+	static variant main_callable_var;
+	static std::vector<variant> v;
+	
+	if(callable == NULL) {
+		callable = new map_formula_callable;
+		callable_var = variant(callable);
+		callable->add("x", variant(0));
+		for(int n = 0; n != 1000; ++n) {
+			v.push_back(callable_var);
+		}
+
+		callable = new map_formula_callable;
+		main_callable_var = variant(callable);
+		callable->add("items", variant(&v));
+	}
+
+	static formula f("map(items, 'obj', 0)");
+	BENCHMARK_LOOP {
+		f.execute(*callable);
+	}
+}
