@@ -21,8 +21,12 @@ public:
 
 	static void set_underwater(bool value);
 	static void set_can_interact(bool value);
+	static void set_on_platform(bool value);
+	static void set_standing(bool value);
 
 	static void draw();
+
+	static void read_controls();
 	
 private:
 	static bool hittest_button (const rect& r);
