@@ -14,12 +14,15 @@
 #include "foreach.hpp"
 #include "formatter.hpp"
 #include "gui_formula_functions.hpp"
+#include "iphone_controls.hpp"
 #include "level.hpp"
 #include "level_object.hpp"
+#include "light.hpp"
 #include "load_level.hpp"
 #include "multiplayer.hpp"
 #include "object_events.hpp"
 #include "player_info.hpp"
+#include "playable_custom_object.hpp"
 #include "preferences.hpp"
 #include "preprocessor.hpp"
 #include "random.hpp"
@@ -27,6 +30,7 @@
 #include "stats.hpp"
 #include "string_utils.hpp"
 #include "surface_palette.hpp"
+#include "texture_frame_buffer.hpp"
 #include "thread.hpp"
 #include "tile_map.hpp"
 #include "unit_test.hpp"
@@ -53,6 +57,10 @@ std::map<std::string, level::summary> load_level_summaries() {
 	return result;
 }
 
+bool level_tile_not_in_rect(const rect& r, const level_tile& t) {
+	return t.x < r.x() || t.y < r.y() || t.x >= r.x2() || t.y >= r.y2();
+}
+
 }
 
 level::summary level::get_summary(const std::string& id)
@@ -76,27 +84,149 @@ void level::set_as_current_level()
 {
 	current_level = this;
 	frame::set_color_palette(palettes_used_);
+
+#if !TARGET_OS_IPHONE
+	static const int starting_x_resolution = preferences::actual_screen_width();
+	static const int starting_y_resolution = preferences::actual_screen_height();
+	static const int starting_virtual_x_resolution = preferences::virtual_screen_width();
+	static const int starting_virtual_y_resolution = preferences::virtual_screen_height();
+
+	if(!editor_ && starting_x_resolution == starting_virtual_x_resolution) {
+		if(!x_resolution_) {
+			x_resolution_ = starting_x_resolution;
+		}
+
+		if(!y_resolution_) {
+			y_resolution_ = starting_y_resolution;
+		}
+
+
+		if(x_resolution_ != preferences::actual_screen_width() || y_resolution_ != preferences::actual_screen_height()) {
+			const bool result = graphics::set_video_mode(x_resolution_, y_resolution_);
+			if(result) {
+				preferences::set_actual_screen_width(x_resolution_);
+				preferences::set_actual_screen_height(y_resolution_);
+				preferences::set_virtual_screen_width(x_resolution_);
+				preferences::set_virtual_screen_height(y_resolution_);
+			}
+		}
+	}
+	
+#endif
+}
+
+namespace {
+graphics::color_transform default_dark_color() {
+	return graphics::color_transform(0, 0, 0, 0);
+}
 }
 
 level::level(const std::string& level_cfg)
-	: id_(level_cfg), highlight_layer_(INT_MIN),
+	: id_(level_cfg),
+	  x_resolution_(0), y_resolution_(0),
+	  highlight_layer_(INT_MIN),
 	  num_compiled_tiles_(0),
 	  entered_portal_active_(false), save_point_x_(-1), save_point_y_(-1),
-	  editor_(false), show_foreground_(true), show_background_(true), air_resistance_(0), water_resistance_(7), end_game_(false),
+	  editor_(false), show_foreground_(true), show_background_(true), dark_(false), dark_color_(graphics::color_transform(0, 0, 0, 255)), air_resistance_(0), water_resistance_(7), end_game_(false),
       editor_tile_updates_frozen_(0), zoom_level_(1),
 	  palettes_used_(0),
-	  background_palette_(-1)
+	  background_palette_(-1),
+	  segment_width_(0), segment_height_(0)
 {
 	std::cerr << "in level constructor...\n";
 	const int start_time = SDL_GetTicks();
 
 	wml::const_node_ptr node = load_level_wml(level_cfg);
+	wml::const_node_ptr player_save_node;
 	ASSERT_LOG(node.get() != NULL, "LOAD LEVEL WML FOR " << level_cfg << " FAILED");
+	if(node->has_attr("id")) {
+		id_ = node->attr("id");
+	}
+
+	if(preferences::load_compiled() && (level_cfg == "save.cfg" || level_cfg == "autosave.cfg")) {
+		if(preferences::version() != node->attr("version").str()) {
+			std::cerr << "DIFFERENT VERSION LEVEL\n";
+			FOREACH_WML_CHILD(obj_node, node, "character") {
+				if(wml::get_bool(obj_node, "is_human", false)) {
+					player_save_node = obj_node;
+					break;
+				}
+			}
+
+			wml::const_node_ptr n = node;
+			if(node->has_attr("id")) {
+				n = load_level_wml(node->attr("id"));
+			} else {
+				//this save was made before we saved level ID's. The best
+				//we can do is get a level with a matching title.
+				std::vector<std::string> files;
+				sys::get_files_in_dir("data/compiled/level/", &files);
+				foreach(const std::string& file, files) {
+					if(file == "save.cfg" || file == "autosave.cfg") {
+						continue;
+					}
+
+					wml::const_node_ptr lvl_info = load_level_wml(file);
+					if(lvl_info->attr("title").str() == node->attr("title").str()) {
+						if(lvl_info->attr("dimensions").str() == node->attr("dimensions").str()) {
+							//the dimensions match so we're sure this is
+							//the right one.
+							n = lvl_info;
+							break;
+						}
+
+						//tenatively guess this is the level, but keep
+						//searching in case one with an exact dimensional
+						//match is found.
+						n = lvl_info;
+					}
+				}
+			}
+			//in case the player had swallowed an object, copy it
+			//over from the savegame.
+			wml::node::const_child_iterator i1 = node->begin_child("serialized_objects");
+			wml::node::const_child_iterator i2 = n->begin_child("serialized_objects");
+			if(i1 != node->end_child("serialized_objects") && i2 != n->end_child("serialized_objects")) {
+				wml::node::const_child_iterator j1 = (i1->second)->begin_child("character");
+				wml::node::const_child_iterator j2 = (i1->second)->end_child("character");
+				while(j1 != j2) {
+					(i2->second)->add_child(j1->second);
+					++j1;
+				}
+			}
+			node = n;
+		}
+	}
+
+	dark_color_ = default_dark_color();
+	if(wml::get_bool(node, "dark", false)) {
+		dark_ = true;
+	}
+
+	if(node->has_attr("dark_color")) {
+		dark_color_ = graphics::color_transform(node->attr("dark_color"));
+	}
+
+	if(node->get_child("vars")) {
+		wml::const_node_ptr vars = node->get_child("vars");
+		for(wml::node::const_attr_iterator i = vars->begin_attr();
+		    i != vars->end_attr(); ++i) {
+			vars_[i->first].serialize_from_string(i->second);
+		}
+	}
+
+	segment_width_ = wml::get_int(node, "segment_width");
+	ASSERT_LOG(segment_width_%32 == 0, "segment_width in " << id_ << " is not divisible by 32");
+
+	segment_height_ = wml::get_int(node, "segment_height");
+	ASSERT_LOG(segment_height_%32 == 0, "segment_height in " << id_ << " is not divisible by 32");
 
 	music_ = node->attr("music");
 	replay_data_ = node->attr("replay_data");
 	cycle_ = wml::get_int(node, "cycle");
 	time_freeze_ = 0;
+	x_resolution_ = wml::get_int(node, "x_resolution");
+	y_resolution_ = wml::get_int(node, "y_resolution");
 	in_dialog_ = false;
 	title_ = node->attr("title");
 	if(node->has_attr("dimensions")) {
@@ -206,10 +336,18 @@ level::level(const std::string& level_cfg)
 	wml::node::const_child_iterator c1 = node->begin_child("character");
 	wml::node::const_child_iterator c2 = node->end_child("character");
 	for(; c1 != c2; ++c1) {
+		if(player_save_node.get() != NULL && wml::get_bool(c1->second, "is_human", false)) {
+			continue;
+		}
+
 		wml_chars_.push_back(c1->second);
 		continue;
 	}
 
+	if(player_save_node.get() != NULL) {
+		wml_chars_.push_back(player_save_node);
+	}
+
 	wml::const_node_ptr serialized_objects = node->get_child("serialized_objects");
 	if(serialized_objects.get() != NULL) {
 		wml_chars_.push_back(serialized_objects);
@@ -269,6 +407,19 @@ level::level(const std::string& level_cfg)
 	gui_algorithm_ = gui_algorithm::get(gui_algo_str_);
 	gui_algorithm_->new_level();
 
+	sub_level_str_ = node->attr("sub_levels");
+	foreach(const std::string& sub_lvl, util::split(node->attr("sub_levels"))) {
+		sub_level_data& data = sub_levels_[sub_lvl];
+		data.lvl = boost::intrusive_ptr<level>(new level(sub_lvl + ".cfg"));
+		foreach(int layer, data.lvl->layers_) {
+			layers_.insert(layer);
+		}
+
+		data.active = false;
+		data.xoffset = data.yoffset = 0;
+		data.xbase = data.ybase = 0;
+	}
+
 }
 
 level::~level()
@@ -352,8 +503,65 @@ void level::load_character(wml::const_node_ptr c)
 
 void level::finish_loading()
 {
+	std::vector<sub_level_data> sub_levels;
+	if((segment_width_ > 0 || segment_height_ > 0) && !editor_ && !preferences::compiling_tiles) {
+
+		const int seg_width = segment_width_ > 0 ? segment_width_ : boundaries_.w();
+		const int seg_height = segment_height_ > 0 ? segment_height_ : boundaries_.h();
+
+		for(int y = boundaries_.y(); y < boundaries_.y2(); y += seg_height) {
+			for(int x = boundaries_.x(); x < boundaries_.x2(); x += seg_width) {
+				level* sub_level = new level(*this);
+				const rect bounds(x, y, seg_width, seg_height);
+
+				sub_level->boundaries_ = bounds;
+				sub_level->tiles_.erase(std::remove_if(sub_level->tiles_.begin(), sub_level->tiles_.end(), boost::bind(level_tile_not_in_rect, bounds, _1)), sub_level->tiles_.end());
+				sub_level->solid_.clear();
+				sub_level->standable_.clear();
+				foreach(const level_tile& t, sub_level->tiles_) {
+					sub_level->add_tile_solid(t);
+				}
+				sub_level->prepare_tiles_for_drawing();
+
+				sub_level_data data;
+				data.lvl.reset(sub_level);
+				data.xbase = x;
+				data.ybase = y;
+				data.xoffset = data.yoffset = 0;
+				data.active = false;
+				sub_levels.push_back(data);
+			}
+		}
+
+		const std::vector<entity_ptr> objects = get_chars();
+		foreach(const entity_ptr& obj, objects) {
+			if(!obj->is_human()) {
+				remove_character(obj);
+			}
+		}
+
+		solid_.clear();
+		standable_.clear();
+		tiles_.clear();
+		prepare_tiles_for_drawing();
+
+		int index = 0;
+		foreach(const sub_level_data& data, sub_levels) {
+			sub_levels_[formatter() << index] = data;
+			++index;
+		}
+	}
+
+	if(sub_levels_.empty() == false) {
+		solid_base_ = solid_;
+		standable_base_ = standable_;
+	}
+
 	graphics::texture::build_textures_from_worker_threads();
 
+	if (editor_ || preferences::compiling_tiles)
+		game_logic::set_verbatim_string_expressions (true);
+
 	game_logic::wml_formula_callable_read_scope read_scope;
 	foreach(wml::const_node_ptr node, wml_chars_) {
 		if(node->name() != "serialized_objects") {
@@ -396,6 +604,8 @@ void level::finish_loading()
 		}
 	}
 
+	game_logic::set_verbatim_string_expressions (false);
+
 	wml_chars_.clear();
 
 	controls::new_level(cycle_, players_.empty() ? 1 : players_.size(), multiplayer::slot());
@@ -408,6 +618,26 @@ void level::finish_loading()
 	if(!next_level().empty()) {
 		preload_level_wml(next_level());
 	}
+
+	if(!sub_levels.empty()) {
+		const int seg_width = segment_width_ > 0 ? segment_width_ : boundaries_.w();
+		const int seg_height = segment_height_ > 0 ? segment_height_ : boundaries_.h();
+		int segment_number = 0;
+		for(int y = boundaries_.y(); y < boundaries_.y2(); y += seg_height) {
+			for(int x = boundaries_.x(); x < boundaries_.x2(); x += seg_width) {
+				const std::vector<entity_ptr> objects = get_chars();
+				foreach(const entity_ptr& obj, objects) {
+					if(!obj->is_human() && obj->midpoint().x >= x && obj->midpoint().x < x + seg_width && obj->midpoint().y >= y && obj->midpoint().y < y + seg_height) {
+						ASSERT_INDEX_INTO_VECTOR(segment_number, sub_levels);
+						sub_levels[segment_number].lvl->add_character(obj);
+						remove_character(obj);
+					}
+				}
+
+				++segment_number;
+			}
+		}
+	}
 }
 
 void level::set_multiplayer_slot(int slot)
@@ -689,21 +919,54 @@ void level::rebuild_tiles_rect(const rect& r)
 	prepare_tiles_for_drawing();
 }
 
+std::string level::package() const
+{
+	std::string::const_iterator i = std::find(id_.begin(), id_.end(), '/');
+	if(i == id_.end()) {
+		return "";
+	}
+
+	return std::string(id_.begin(), i);
+}
+
 wml::node_ptr level::write() const
 {
 	std::sort(tiles_.begin(), tiles_.end(), level_tile_zorder_pos_comparer());
 	game_logic::wml_formula_callable_serialization_scope serialization_scope;
 
 	wml::node_ptr res(new wml::node("level"));
+	res->set_attr("id", id_);
+	res->set_attr("version", preferences::version());
 	res->set_attr("title", title_);
 	res->set_attr("music", music_);
+	res->set_attr("segment_width", formatter() << segment_width_);
+	res->set_attr("segment_height", formatter() << segment_height_);
+
+	if(x_resolution_ || y_resolution_) {
+		res->set_attr("x_resolution", formatter() << x_resolution_);
+		res->set_attr("y_resolution", formatter() << y_resolution_);
+	}
+
 	if(gui_algo_str_ != "default") {
 		res->set_attr("gui", gui_algo_str_);
 	}
 
+	if(dark_) {
+		res->set_attr("dark", "yes");
+	}
+
+	if(dark_color_.to_string() != default_dark_color().to_string()) {
+		res->set_attr("dark_color", dark_color_.to_string());
+	}
+
 	if(cycle_) {
 		res->set_attr("cycle", formatter() << cycle_);
 	}
+
+	if(!sub_level_str_.empty()) {
+		res->set_attr("sub_levels", sub_level_str_);
+	}
+
 	res->set_attr("dimensions", boundaries().to_string());
 
 	res->set_attr("xscale", formatter() << xscale_);
@@ -1006,6 +1269,18 @@ wml::node_ptr level::write() const
 		res->set_attr("background_palette", graphics::get_palette_name(background_palette_));
 	}
 
+	if(!vars_.empty()) {
+		wml::node_ptr vars(new wml::node("vars"));
+		for(std::map<std::string, variant>::const_iterator i = vars_.begin();
+		    i != vars_.end(); ++i) {
+			std::string value;
+			i->second.serialize_to_string(value);
+			vars->set_attr(i->first, value);
+		}
+
+		res->add_child(vars);
+	}
+
 	return res;
 }
 
@@ -1061,6 +1336,15 @@ void level::draw_layer(int layer, int x, int y, int w, int h) const
 		return;
 	}
 
+	for(std::map<std::string, sub_level_data>::const_iterator i = sub_levels_.begin(); i != sub_levels_.end(); ++i) {
+		if(i->second.active) {
+			glPushMatrix();
+			glTranslatef(i->second.xoffset, i->second.yoffset, 0.0);
+			i->second.lvl->draw_layer(layer, x - i->second.xoffset, y - i->second.yoffset - TileSize, w, h + TileSize);
+			glPopMatrix();
+		}
+	}
+
 	if(editor_ && layer == highlight_layer_) {
 		const GLfloat alpha = 0.3 + (1.0+sin(draw_count/5.0))*0.35;
 		glColor4f(1.0, 1.0, 1.0, alpha);
@@ -1242,7 +1526,11 @@ void level::prepare_tiles_for_drawing()
 	blit_cache_.clear();
 
 	for(int n = 0; n != tiles_.size(); ++n) {
-		if(tiles_[n].object->solid_color()) {
+		if(!is_arcade_level() && tiles_[n].object->solid_color()) {
+			continue;
+		}
+
+		if(!editor_ && (tiles_[n].x <= boundaries().x() - TileSize || tiles_[n].y <= boundaries().y() - TileSize || tiles_[n].x >= boundaries().x2() || tiles_[n].y >= boundaries().y2())) {
 			continue;
 		}
 
@@ -1263,8 +1551,11 @@ void level::prepare_tiles_for_drawing()
 	}
 
 	for(int n = 0; n != tiles_.size(); ++n) {
+		if(!editor_ && (tiles_[n].x <= boundaries().x() - TileSize || tiles_[n].y <= boundaries().y() - TileSize || tiles_[n].x >= boundaries().x2() || tiles_[n].y >= boundaries().y2())) {
+			continue;
+		}
 
-		if(tiles_[n].object->solid_color()) {
+		if(!is_arcade_level() && tiles_[n].object->solid_color()) {
 			tiles_[n].draw_disabled = true;
 			if(!solid_color_rects_.empty()) {
 				solid_color_rect& r = solid_color_rects_.back();
@@ -1351,9 +1642,9 @@ void level::prepare_tiles_for_drawing()
 namespace {
 bool sort_entity_drawing_pos(const entity_ptr& a, const entity_ptr& b) {
 	return a->zorder() < b->zorder() ||
-	       a->zorder() == b->zorder() && a->y() < b->y() ||
-		   a->zorder() == b->zorder() && a->y() < b->y() ||
-		   a->zorder() == b->zorder() && a->y() == b->y() && a->x() < b->x();
+		   a->zorder() == b->zorder() && a->zsub_order() < b->zsub_order() ||
+		   a->zorder() == b->zorder() && a->zsub_order() == b->zsub_order() && a->y() < b->y() ||
+		   a->zorder() == b->zorder() && a->zsub_order() == b->zsub_order() && a->y() == b->y() && a->x() < b->x();
 }
 }
 
@@ -1361,6 +1652,7 @@ void level::draw_status() const
 {
 	if(gui_algorithm_) {
 		gui_algorithm_->draw(*this);
+		iphone_controls::draw();
 	}
 
 	if(current_speech_dialog()) {
@@ -1368,6 +1660,32 @@ void level::draw_status() const
 	}
 }
 
+namespace {
+void draw_entity(const entity& obj, int x, int y, bool editor) {
+	const std::pair<int,int>* scroll_speed = obj.position_scale_millis();
+
+	if(scroll_speed) {
+		glPushMatrix();
+		const int scrollx = scroll_speed->first;
+		const int scrolly = scroll_speed->second;
+
+		const int diffx = ((scrollx - 1000)*x)/1000;
+		const int diffy = ((scrolly - 1000)*y)/1000;
+
+		glTranslatef(diffx, diffy, 0.0);
+	}
+
+	obj.draw();
+	if(editor) {
+		obj.draw_group();
+	}
+
+	if(scroll_speed) {
+		glPopMatrix();
+	}
+}
+}
+
 extern std::vector<rect> background_rects_drawn;
 
 void level::draw(int x, int y, int w, int h) const
@@ -1376,6 +1694,8 @@ void level::draw(int x, int y, int w, int h) const
 
 	const int start_x = x;
 	const int start_y = y;
+	const int start_w = w;
+	const int start_h = h;
 
 	const int ticks = SDL_GetTicks();
 	x -= widest_tile_;
@@ -1398,6 +1718,14 @@ void level::draw(int x, int y, int w, int h) const
 
 	std::vector<entity_ptr>::const_iterator entity_itor = chars.begin();
 
+	
+	/*std::cerr << "SUMMARY " << cycle_ << ": ";
+	foreach(const entity_ptr& e, chars_) {
+		std::cerr << e->debug_description() << "(" << e->zsub_order() << "):";
+	}
+	
+	std::cerr << "\n";*/
+	
 	bool water_drawn = true;
 	int water_zorder = 0;
 	if(water_) {
@@ -1407,52 +1735,6 @@ void level::draw(int x, int y, int w, int h) const
 
 	std::set<int>::const_iterator layer = layers_.begin();
 
-	int player_zorder = 0;
-	if(players_.empty() == false) {
-		player_zorder = players_.front()->zorder();
-	}
-
-	for(; layer != layers_.end() && *layer < player_zorder; ++layer) {
-
-		if(!water_drawn && *layer > water_zorder) {
-			water_->draw(x, y, w, h);
-			water_drawn = true;
-		}
-
-		while(entity_itor != chars.end() && (*entity_itor)->zorder() <= *layer) {
-			if(!(*entity_itor)->is_human()) {
-				const std::pair<int,int>* scroll_speed = (*entity_itor)->position_scale_millis();
-
-				if(scroll_speed) {
-					glPushMatrix();
-					const int scrollx = scroll_speed->first;
-					const int scrolly = scroll_speed->second;
-
-					const int diffx = ((scrollx - 1000)*x)/1000;
-					const int diffy = ((scrolly - 1000)*y)/1000;
-
-					glTranslatef(diffx, diffy, 0.0);
-				}
-
-				(*entity_itor)->draw();
-				if(editor_) {
-					(*entity_itor)->draw_group();
-				}
-
-				if(scroll_speed) {
-					glPopMatrix();
-				}
-			}
-			++entity_itor;
-		}
-
-		draw_layer(*layer, x, y, w, h);
-	}
-
-	foreach(const const_entity_ptr& p, players_) {
-		p->draw();
-	}
-
 	for(; layer != layers_.end(); ++layer) {
 		if(!water_drawn && *layer > water_zorder) {
 			water_->draw(x, y, w, h);
@@ -1460,12 +1742,7 @@ void level::draw(int x, int y, int w, int h) const
 		}
 
 		while(entity_itor != chars.end() && (*entity_itor)->zorder() <= *layer) {
-			if(!(*entity_itor)->is_human()) {
-				(*entity_itor)->draw();
-				if(editor_) {
-					(*entity_itor)->draw_group();
-				}
-			}
+			draw_entity(**entity_itor, x, y, editor_);
 			++entity_itor;
 		}
 
@@ -1478,13 +1755,7 @@ void level::draw(int x, int y, int w, int h) const
 	}
 
 	while(entity_itor != chars.end()) {
-		if(!(*entity_itor)->is_human()) {
-			(*entity_itor)->draw();
-			if(editor_) {
-
-				(*entity_itor)->draw_group();
-			}
-		}
+		draw_entity(**entity_itor, x, y, editor_);
 		++entity_itor;
 	}
 
@@ -1525,6 +1796,50 @@ void level::draw(int x, int y, int w, int h) const
 	if(background_) {
 		background_->draw_foreground(start_x, start_y, 0.0, cycle());
 	}
+
+	calculate_lighting(start_x, start_y, start_w, start_h);
+}
+
+void level::calculate_lighting(int x, int y, int w, int h) const
+{
+	if(!dark_ || editor_ || texture_frame_buffer::unsupported()) {
+		return;
+	}
+
+	//find all the lights in the level
+	static std::vector<const light*> lights;
+	lights.clear();
+	foreach(const entity_ptr& c, active_chars_) {
+		foreach(const light_ptr& lt, c->lights()) {
+			lights.push_back(lt.get());
+		}
+	}
+
+	{
+		glBlendFunc(GL_ONE, GL_ONE);
+		rect screen_area(x, y, w, h);
+		const texture_frame_buffer::render_scope scope;
+
+		glClearColor(dark_color_.r()/255.0, dark_color_.g()/255.0, dark_color_.b()/255.0, dark_color_.a()/255.0);
+		glClear(GL_COLOR_BUFFER_BIT);
+		const unsigned char color[] = { dark_color_.r(), dark_color_.g(), dark_color_.b(), dark_color_.a() };
+		foreach(const light* lt, lights) {
+			lt->draw(screen_area, color);
+		}
+	}
+
+	//now blit the light buffer onto the screen
+	texture_frame_buffer::set_as_current_texture();
+
+	const GLfloat tcarray[] = { 0, 0, 0, 1, 1, 0, 1, 1 };
+	const GLfloat tcarray_rotated[] = { 0, 1, 1, 1, 0, 0, 1, 0 };
+	GLfloat varray[] = { x, y + h, x, y, x + w, y + h, x + w, y };
+	glVertexPointer(2, GL_FLOAT, 0, varray);
+	glTexCoordPointer(2, GL_FLOAT, 0,
+	               preferences::screen_rotated() ? tcarray_rotated : tcarray);
+	glBlendFunc(GL_ONE_MINUS_SRC_ALPHA, GL_SRC_ALPHA);
+	glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
+	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 }
 
 void level::draw_debug_solid(int x, int y, int w, int h) const
@@ -1636,8 +1951,6 @@ void level::process()
 		gui_algorithm_->process(*this);
 	}
 
-	multiplayer::send_and_receive();
-
 	const int LevelPreloadFrequency = 500; //10 seconds
 	//see if we have levels to pre-load. Load one periodically.
 	if((cycle_%LevelPreloadFrequency) == 0) {
@@ -1649,10 +1962,12 @@ void level::process()
 
 	controls::read_local_controls();
 
+	multiplayer::send_and_receive();
+
 	do_processing();
 
 	if(speech_dialogs_.empty() == false) {
-		if(speech_dialogs_.front()->process()) {
+		if(speech_dialogs_.top()->process()) {
 			speech_dialogs_.pop();
 		}
 	}
@@ -1705,6 +2020,7 @@ void level::do_processing()
 		} else { //char is inactive
 			if( c->dies_on_inactive() ){
 				if(c->label().empty() == false) {
+					c->die_with_no_event();
 					chars_by_label_.erase(c->label());
 				}
 				
@@ -1718,6 +2034,22 @@ void level::do_processing()
 	std::sort(active_chars_.begin(), active_chars_.end());
 	active_chars_.erase(std::unique(active_chars_.begin(), active_chars_.end()), active_chars_.end());
 	std::sort(active_chars_.begin(), active_chars_.end(), sort_entity_drawing_pos);
+	
+/*
+	std::cerr << "SUMMARY " << cycle_ << ": ";
+	foreach(const entity_ptr& e, chars_) {
+		std::cerr << e->debug_description() << "(" << (e->is_human() ? "HUMAN," : "") << e->centi_x() << "," << e->centi_y() << "):";
+	}
+
+	std::cerr << "\n";
+	*/
+
+	int checksum = 0;
+	foreach(const entity_ptr& e, chars_) {
+		checksum += e->x() + e->y();
+	}
+
+	controls::set_checksum(cycle_, checksum);
 
 	const int ActivationDistance = 700;
 
@@ -1900,6 +2232,24 @@ bool level::is_solid(const level_solid_map& map, int x, int y, int* friction, in
 	return false;
 }
 
+bool level::standable(const rect& r, int* friction, int* traction, int* damage) const
+{
+	const int ybegin = r.y();
+	const int yend = r.y2();
+	const int xbegin = r.x();
+	const int xend = r.x2();
+
+	for(int y = ybegin; y != yend; ++y) {
+		for(int x = xbegin; x != xend; ++x) {
+			if(standable(x, y, friction, traction, damage)) {
+				return true;
+			}
+		}
+	}
+
+	return false;
+}
+
 bool level::standable(int x, int y, int* friction, int* traction, int* damage) const
 {
 	if(is_solid(solid_, x, y, friction, traction, damage) ||
@@ -2276,12 +2626,12 @@ const level_tile* level::get_tile_at(int x, int y) const
 
 void level::remove_character(entity_ptr e)
 {
-	std::cerr << "removing char: '" << e->label() << "'\n";
+	//std::cerr << "removing char: '" << e->label() << "'\n";
 	if(e->label().empty() == false) {
 		chars_by_label_.erase(e->label());
 	}
 	chars_.erase(std::remove(chars_.begin(), chars_.end(), e), chars_.end());
-	std::cerr << "removed char: '" << e->label() << "'\n";
+	//std::cerr << "removed char: '" << e->label() << "'\n";
 }
 
 std::vector<entity_ptr> level::get_characters_in_rect(const rect& r) const
@@ -2296,17 +2646,42 @@ std::vector<entity_ptr> level::get_characters_in_rect(const rect& r) const
 	return res;
 }
 
-entity_ptr level::get_character_at_point(int x, int y) const
+std::vector<entity_ptr> level::get_characters_at_point(int x, int y) const
 {
+	std::vector<entity_ptr> result;
 	foreach(entity_ptr c, chars_) {
 		const int xpos = x - c->x();
 		const int ypos = y - c->y();
 		if(!c->is_alpha(x, y)) {
-			return c;
+			result.push_back(c);
 		}
 	}
 	
-	return entity_ptr();
+	return result;
+}
+
+entity_ptr level::get_next_character_at_point(int x, int y) const
+{
+	std::vector<entity_ptr> v = get_characters_at_point(x, y);
+	if(v.empty()) {
+		return entity_ptr();
+	}
+
+	if(editor_selection_.empty()) {
+		return v.front();
+	}
+
+	std::vector<entity_ptr>::iterator itor = std::find(v.begin(), v.end(), editor_selection_.back());
+	if(itor == v.end()) {
+		return v.front();
+	}
+
+	++itor;
+	if(itor == v.end()) {
+		itor = v.begin();
+	}
+
+	return *itor;
 }
 
 void level::add_solid_rect(int x1, int y1, int x2, int y2, int friction, int traction, int damage)
@@ -2386,6 +2761,18 @@ void level::set_solid(level_solid_map& map, int x, int y, int friction, int trac
 	}
 }
 
+void level::add_multi_player(entity_ptr p)
+{
+	last_touched_player_ = p;
+	p->get_player_info()->set_player_slot(players_.size());
+	players_.push_back(p);
+	chars_.push_back(p);
+	if(p->label().empty() == false) {
+		chars_by_label_[p->label()] = p;
+	}
+	layers_.insert(p->zorder());
+}
+
 void level::add_player(entity_ptr p)
 {
 	chars_.erase(std::remove(chars_.begin(), chars_.end(), player_), chars_.end());
@@ -2413,6 +2800,13 @@ void level::add_player(entity_ptr p)
 		}
 	}
 
+	const int difficulty = current_difficulty();
+	for(int n = 0; n != chars_.size(); ++n) {
+		if(chars_[n].get() != NULL && !chars_[n]->appears_at_difficulty(difficulty)) {
+			chars_[n] = entity_ptr();
+		}
+	}
+
 	chars_.erase(std::remove(chars_.begin(), chars_.end(), entity_ptr()), chars_.end());
 }
 
@@ -2538,15 +2932,15 @@ namespace {
 
 const std::string LevelProperties[] = {
   "cycle", "player", "in_dialog",
-  "local_player", "num_active", "active_chars", "chars",
-  "in_editor", "zoom", "focus", "gui", "id",
+  "local_player", "num_active", "active_chars", "chars", "players",
+  "in_editor", "zoom", "focus", "gui", "id", "dimensions",
 };
 
 enum LEVEL_PROPERTY_ID {
 	LEVEL_CYCLE, LEVEL_PLAYER, LEVEL_IN_DIALOG,
 	LEVEL_LOCAL_PLAYER, LEVEL_NUM_ACTIVE,
-	LEVEL_ACTIVE_CHARS, LEVEL_CHARS, LEVEL_IN_EDITOR, LEVEL_ZOOM,
-	LEVEL_FOCUS, LEVEL_GUI, LEVEL_ID
+	LEVEL_ACTIVE_CHARS, LEVEL_CHARS, LEVEL_PLAYERS, LEVEL_IN_EDITOR, LEVEL_ZOOM,
+	LEVEL_FOCUS, LEVEL_GUI, LEVEL_ID, LEVEL_DIMENSIONS,
 };
 }
 
@@ -2585,6 +2979,14 @@ variant level::get_value_by_slot(int slot) const
 
 		return variant(&v);
 	}
+	case LEVEL_PLAYERS: {
+		std::vector<variant> v;
+		foreach(const entity_ptr& e, players()) {
+			v.push_back(variant(e.get()));
+		}
+
+		return variant(&v);
+	}
 	case LEVEL_IN_EDITOR:
 		return variant(editor_);
 	case LEVEL_ZOOM:
@@ -2607,6 +3009,14 @@ variant level::get_value_by_slot(int slot) const
 	case LEVEL_ID: {
 		return variant(id_);
 	}
+	case LEVEL_DIMENSIONS: {
+		std::vector<variant> v;
+		v.push_back(variant(boundaries_.x()));
+		v.push_back(variant(boundaries_.y()));
+		v.push_back(variant(boundaries_.x2()));
+		v.push_back(variant(boundaries_.y2()));
+		return variant(&v);
+	}
 	}
 
 	ASSERT_LOG(false, "BAD SLOT IN GET_VALUE FROM LEVEL " << slot);
@@ -2667,6 +3077,27 @@ variant level::get_value(const std::string& key) const
 		return variant(&v);
 	} else if(key == "id") {
 		return variant(id());
+	} else if(key == "dimensions") {
+		std::vector<variant> v;
+		v.push_back(variant(boundaries_.x()));
+		v.push_back(variant(boundaries_.y()));
+		v.push_back(variant(boundaries_.x2()));
+		v.push_back(variant(boundaries_.y2()));
+		return variant(&v);
+	} else if(key == "segment_width") {
+		return variant(segment_width_);
+	} else if(key == "segment_height") {
+		return variant(segment_height_);
+	} else if(key == "num_segments") {
+		return variant(sub_levels_.size());
+	} else if(key == "camera_position") {
+		std::vector<variant> pos;
+		pos.reserve(4);
+		pos.push_back(variant(last_draw_position().x/100));
+		pos.push_back(variant(last_draw_position().y/100));
+		pos.push_back(variant(graphics::screen_width()));
+		pos.push_back(variant(graphics::screen_height()));
+		return variant(&pos);
 	} else {
 		const_entity_ptr e = get_entity_by_label(key);
 		if(e) {
@@ -2714,6 +3145,9 @@ void level::set_value(const std::string& key, const variant& value)
 				chars_immune_from_time_freeze_.push_back(e);
 			}
 		}
+	} else if(key == "dimensions") {
+		ASSERT_EQ(value.num_elements(), 4);
+		boundaries_ = rect(value[0].as_int(), value[1].as_int(), value[2].as_int() - value[0].as_int(), value[3].as_int() - value[1].as_int());
 	} else {
 		vars_[key] = value;
 	}
@@ -2851,6 +3285,7 @@ bool level::can_interact(const rect& body) const
 
 void level::replay_from_cycle(int ncycle)
 {
+	return;
 	const int cycles_ago = cycle_ - ncycle;
 	if(cycles_ago <= 0) {
 		return;
@@ -2871,6 +3306,8 @@ void level::replay_from_cycle(int ncycle)
 
 void level::backup()
 {
+	return;
+
 	std::map<entity_ptr, entity_ptr> entity_map;
 
 	std::cerr << "BACKUP " << cycle_ << ": ";
@@ -2878,6 +3315,7 @@ void level::backup()
 	snapshot->rng_seed = rng::get_seed();
 	snapshot->cycle = cycle_;
 	snapshot->chars.reserve(chars_.size());
+
 	foreach(const entity_ptr& e, chars_) {
 		std::cerr << e->debug_description() << "(" << (e->is_human() ? "HUMAN," : "") << e->centi_x() << "," << e->centi_y() << "):";
 		snapshot->chars.push_back(e->backup());
@@ -2963,13 +3401,20 @@ void level::add_speech_dialog(boost::shared_ptr<speech_dialog> d)
 	speech_dialogs_.push(d);
 }
 
+void level::remove_speech_dialog()
+{
+	if(speech_dialogs_.empty() == false) {
+		speech_dialogs_.pop();
+	}
+}
+
 boost::shared_ptr<const speech_dialog> level::current_speech_dialog() const
 {
 	if(speech_dialogs_.empty()) {
 		return boost::shared_ptr<const speech_dialog>();
 	}
 
-	return speech_dialogs_.front();
+	return speech_dialogs_.top();
 }
 
 bool entity_in_current_level(const entity* e)
@@ -2978,6 +3423,187 @@ bool entity_in_current_level(const entity* e)
 	return std::find(lvl.get_chars().begin(), lvl.get_chars().end(), e) != lvl.get_chars().end();
 }
 
+void level::add_sub_level(const std::string& lvl, int xoffset, int yoffset, bool add_objects)
+{
+
+	const std::map<std::string, sub_level_data>::iterator itor = sub_levels_.find(lvl);
+	ASSERT_LOG(itor != sub_levels_.end(), "SUB LEVEL NOT FOUND: " << lvl);
+
+	if(itor->second.active && add_objects) {
+		remove_sub_level(lvl);
+	}
+
+	const int xdiff = xoffset - itor->second.xoffset;
+	const int ydiff = yoffset - itor->second.yoffset;
+
+	itor->second.xoffset = xoffset - itor->second.xbase;
+	itor->second.yoffset = yoffset - itor->second.ybase;
+
+	std::cerr << "ADDING SUB LEVEL: " << lvl << "(" << itor->second.lvl->boundaries() << ") " << itor->second.xbase << ", " << itor->second.ybase << " -> " << itor->second.xoffset << ", " << itor->second.yoffset << "\n";
+
+	itor->second.active = true;
+	level& sub = *itor->second.lvl;
+
+	if(add_objects) {
+		const int difficulty = current_difficulty();
+		foreach(entity_ptr e, sub.chars_) {
+			if(e->is_human()) {
+				continue;
+			}
+	
+			entity_ptr c = e->clone();
+			if(!c) {
+				continue;
+			}
+
+			relocate_object(c, c->x() + itor->second.xoffset, c->y() + itor->second.yoffset);
+			if(c->appears_at_difficulty(difficulty)) {
+				add_character(c);
+				c->handle_event(OBJECT_EVENT_START_LEVEL);
+
+				itor->second.objects.push_back(c);
+			}
+		}
+	}
+
+	foreach(solid_color_rect& r, sub.solid_color_rects_) {
+		r.area = rect(r.area.x() + xdiff, r.area.y() + ydiff, r.area.w(), r.area.h());
+	}
+
+	build_solid_data_from_sub_levels();
+}
+
+void level::remove_sub_level(const std::string& lvl)
+{
+	const std::map<std::string, sub_level_data>::iterator itor = sub_levels_.find(lvl);
+	ASSERT_LOG(itor != sub_levels_.end(), "SUB LEVEL NOT FOUND: " << lvl);
+
+	if(itor->second.active) {
+		foreach(entity_ptr& e, itor->second.objects) {
+			if(std::find(active_chars_.begin(), active_chars_.end(), e) == active_chars_.end()) {
+				remove_character(e);
+			}
+		}
+
+		itor->second.objects.clear();
+	}
+
+	itor->second.active = false;
+}
+
+void level::build_solid_data_from_sub_levels()
+{
+	solid_ = solid_base_;
+	standable_ = standable_base_;
+	solid_.clear();
+	standable_.clear();
+
+	for(std::map<std::string, sub_level_data>::const_iterator i = sub_levels_.begin(); i != sub_levels_.end(); ++i) {
+		if(!i->second.active) {
+			continue;
+		}
+
+		const int xoffset = i->second.xoffset/TileSize;
+		const int yoffset = i->second.yoffset/TileSize;
+		solid_.merge(i->second.lvl->solid_, xoffset, yoffset);
+		standable_.merge(i->second.lvl->standable_, xoffset, yoffset);
+	}
+}
+
+void level::adjust_level_offset(int xoffset, int yoffset)
+{
+	game_logic::map_formula_callable* callable(new game_logic::map_formula_callable);
+	variant holder(callable);
+	callable->add("xshift", variant(xoffset));
+	callable->add("yshift", variant(yoffset));
+	foreach(entity_ptr e, chars_) {
+		e->shift_position(xoffset, yoffset);
+		e->handle_event(OBJECT_EVENT_COSMIC_SHIFT, callable);
+	}
+
+	boundaries_ = rect(boundaries_.x() + xoffset, boundaries_.y() + yoffset, boundaries_.w(), boundaries_.h());
+
+	for(std::map<std::string, sub_level_data>::iterator i = sub_levels_.begin();
+	    i != sub_levels_.end(); ++i) {
+		if(i->second.active) {
+			add_sub_level(i->first, i->second.xoffset + xoffset + i->second.xbase, i->second.yoffset + yoffset + i->second.ybase, false);
+		}
+	}
+
+	last_draw_position().x += xoffset*100;
+	last_draw_position().y += yoffset*100;
+	last_draw_position().focus_x += xoffset;
+	last_draw_position().focus_y += yoffset;
+}
+
+bool level::relocate_object(entity_ptr e, int new_x, int new_y)
+{
+	const int orig_x = e->x();
+	const int orig_y = e->y();
+
+	const int delta_x = new_x - orig_x;
+	const int delta_y = new_y - orig_y;
+
+	e->set_pos(new_x, new_y);
+
+	if(!place_entity_in_level(*this, *e)) {
+		//if we can't place the object due to solidity, then cancel
+		//the movement.
+		e->set_pos(orig_x, orig_y);
+		return false;
+	}
+
+
+	//update any x/y co-ordinates to be the same relative to the object's
+	//new position.
+	if(e->editor_info()) {
+		foreach(const editor_variable_info& var, e->editor_info()->vars()) {
+			const variant value = e->query_value(var.variable_name());
+			switch(var.type()) {
+			case editor_variable_info::XPOSITION:
+				if(value.is_int()) {
+					e->handle_event("editor_changing_variable");
+					e->mutate_value(var.variable_name(), variant(value.as_int() + delta_x));
+					e->handle_event("editor_changed_variable");
+				}
+				break;
+			case editor_variable_info::YPOSITION:
+				if(value.is_int()) {
+					e->handle_event("editor_changing_variable");
+					e->mutate_value(var.variable_name(), variant(value.as_int() + delta_y));
+					e->handle_event("editor_changed_variable");
+				}
+				break;
+			default:
+				break;
+			}
+		}
+	}
+
+	return true;
+}
+
+void level::record_zorders()
+{
+	foreach(const level_tile& t, tiles_) {
+		t.object->record_zorder(t.zorder);
+	}
+}
+
+int level::current_difficulty() const
+{
+	if(!last_touched_player_) {
+		return 0;
+	}
+
+	playable_custom_object* p = dynamic_cast<playable_custom_object*>(last_touched_player_.get());
+	if(!p) {
+		return 0;
+	}
+
+	return p->difficulty();
+}
+
 UTILITY(correct_solidity)
 {
 	std::vector<std::string> files;
@@ -3020,6 +3646,7 @@ UTILITY(compile_levels)
 		std::cerr << "LOADING LEVEL '" << file << "'\n";
 		boost::intrusive_ptr<level> lvl(new level(file));
 		lvl->finish_loading();
+		lvl->record_zorders();
 		std::string data;
 		wml::write(lvl->write(), data);
 		sys::write_file("data/compiled/level/" + file, data);
