@@ -2,8 +2,13 @@
 #ifndef SDL_VIDEO_OPENGL_ES
 #include <GL/glew.h>
 #endif
+#ifdef TARGET_PANDORA
+#include <GLES/gl.h>
+#include <GLES/glues.h>
+#else
 #include <GL/gl.h>
 #include <GL/glu.h>
+#endif
 #if defined(__APPLE__) && !(TARGET_IPHONE_SIMULATOR || TARGET_OS_IPHONE)
 #include <OpenGL/OpenGL.h>
 #endif
@@ -32,8 +37,10 @@
 #include "framed_gui_element.hpp"
 #include "graphical_font.hpp"
 #include "gui_section.hpp"
+#include "i18n.hpp"
 #include "inventory.hpp"
 #include "iphone_device_info.h"
+#include "of_bridge.h"
 #include "joystick.hpp"
 #include "key.hpp"
 #include "level.hpp"
@@ -52,6 +59,7 @@
 #include "string_utils.hpp"
 #include "surface_cache.hpp"
 #include "texture.hpp"
+#include "texture_frame_buffer.hpp"
 #include "tile_map.hpp"
 #include "unit_test.hpp"
 #include "wml_node.hpp"
@@ -60,6 +68,10 @@
 #include "wml_utils.hpp"
 #include "wml_writer.hpp"
 
+#if defined(TARGET_PANDORA)
+#include "eglport.h"
+#endif
+
 namespace {
 
 bool show_title_screen(std::string& level_cfg)
@@ -68,15 +80,85 @@ bool show_title_screen(std::string& level_cfg)
 	return false;
 }
 
+void print_help(const std::string& argv0)
+{
+	std::cout << "Usage: " << argv0 << " [OPTIONS]\n" <<
+"\n" <<
+"User options:\n" <<
+//"      --bigscreen              FIXME\n" <<
+"      --config-path=PATH       sets the path to the user config dir\n" <<
+"      --fullscreen             starts in fullscreen mode\n" <<
+"      --height NUM             sets the game window height to which contents\n" <<
+"                                 are scaled\n" <<
+"      --host                   set the game server host address\n" <<
+"      --[no-]joystick          enables/disables joystick support\n" <<
+"      --level LEVEL_FILE       starts the game using the specified level file,\n" <<
+"                                 relative to the level path\n" <<
+"      --level-path=PATH        sets the path to the game level files\n" <<
+"      --[no-]music             enables/disables game music\n" <<
+"      --relay                  use the server as a relay in multiplayer rather\n" <<
+"                                 than trying to initiate direct connections\n" <<
+"      --[no-]resizable         allows/disallows to resize the game window\n" <<
+"      --scale                  enables an experimental pixel art interpolation\n" <<
+"                                 algorithm for scaling the game graphics (some\n" <<
+"                                 issues with this still have to be solved)\n" <<
+"      --no-send-stats          disables sending game statistics over the\n" <<
+"                                 network\n" <<
+"      --[no-]sound             enables/disables sound and music support\n" <<
+"      --widescreen             sets widescreen mode, increasing the game view\n" <<
+"                                 area for wide screen displays\n" <<
+"      --width NUM              sets the game window width to which contents are\n" <<
+"                                 scaled\n" <<
+"      --windowed               starts in windowed mode\n" <<
+"      --wvga                   sets the display size to 800x480\n" <<
+"\n" <<
+"Diagnostic options:\n" <<
+"      --[no-]debug             enables/disables debug mode\n" <<
+"      --[no-]fps               enables/disables framerate display\n" <<
+"      --set-fps=FPS            sets the framerate to FPS\n" <<
+"      --potonly                use power of two-sized textures only\n" <<
+"      --textures16             use 16 bpp textures only (default on iPhone)\n" <<
+"      --textures32             use 32 bpp textures (default on PC/Mac)\n" <<
+
+"\n" <<
+"Developer options:\n" <<
+"      --benchmarks             runs all the engine's benchmarks (intended to\n" <<
+"                                 measure the speed of certain low-level\n" <<
+"                                 functions), only useful if you're actually\n" <<
+"                                 hacking on the engine to optimize the speed\n" <<
+"                                 of these\n" <<
+"      --benchmarks=NAME        runs a single named benchmark code\n" <<
+"      --[no-]compiled          enable or disable precompiled game data\n" <<
+//"      --profile                FIXME\n" <<
+//"      --profile=FILE           FIXME\n" <<
+"      --show-hitboxes          turns on the display of object hitboxes\n" <<
+"      --show-controls          turns on the display of iPhone control hitboxes\n" <<
+"      --simipad                changes various options to emulate an iPad\n" <<
+"                                 environment\n" <<
+"      --simiphone              changes various options to emulate an iPhone\n" <<
+"                                 environment\n" <<
+"      --tests                  runs the game's unit tests and exits\n" <<
+"      --no-tests               skips the execution of unit tests on startup\n"
+"      --utility=NAME           runs the specified UTILITY( NAME ) code block,\n" <<
+"                                 such as compile_levels or object_compiler,\n" <<
+"                                 with the specified arguments\n"
+	;
+}
+
 }
 
 extern "C" int main(int argc, char** argv)
 {
+#if defined(TARGET_PANDORA)
+    EGL_Open();
+#endif
+
 	if(SDL_Init(SDL_INIT_VIDEO|SDL_INIT_JOYSTICK) < 0) {
 		std::cerr << "could not init SDL\n";
 		return -1;
 	}
 
+	i18n::init ();
 
 //	SDL_GL_SetAttribute(SDL_GL_STENCIL_SIZE, 1);
 
@@ -84,6 +166,9 @@ extern "C" int main(int argc, char** argv)
 	std::freopen("/dev/null", "w", stderr);
 	std::cerr.sync_with_stdio(true);
 	#endif
+
+	std::cerr << "Frogatto engine version " << preferences::version() << "\n";
+
 	std::string level_cfg = "titlescreen.cfg";
 	bool unit_tests_only = false, skip_tests = false;
 	bool run_benchmarks = false;
@@ -143,11 +228,20 @@ extern "C" int main(int argc, char** argv)
 			preferences::set_load_compiled(true);
 		} else if(arg == "--no-compiled") {
 			preferences::set_load_compiled(false);
+#if defined(TARGET_PANDORA)
+		} else if(arg == "--no-fbo") {
+			preferences::set_fbo(false);
+		} else if(arg == "--no-bequ") {
+			preferences::set_bequ(false);
+#endif
+		} else if(arg == "--help" || arg == "-h") {
+			print_help(std::string(argv[0]));
+			return 0;
 		} else {
 			const bool res = preferences::parse_arg(argv[n]);
 			if(!res) {
 				std::cerr << "unrecognized arg: '" << arg << "'\n";
-				return 0;
+				return -1;
 			}
 		}
 	}
@@ -161,6 +255,8 @@ extern "C" int main(int argc, char** argv)
 		std::cerr << "cannot create preferences dir!\n";
 	}
 
+	std::cerr << "\n";
+
 #ifdef TARGET_OS_IPHONE
 	//on the iPhone, try to restore the auto-save if it exists
 	if(sys::file_exists(preferences::auto_save_file_path()) && sys::read_file(std::string(preferences::auto_save_file_path()) + ".stat") == "1") {
@@ -188,6 +284,7 @@ extern "C" int main(int argc, char** argv)
 	}
 	preferences::set_virtual_screen_width(height*multiplier);
 	preferences::set_virtual_screen_height(width*multiplier);
+	preferences::set_control_scheme(height % 1024 ? "iphone_2d" : "ipad_2d");
 	
 	SDL_WindowID windowID = SDL_CreateWindow (NULL, 0, 0, preferences::actual_screen_width(), preferences::actual_screen_height(),
 		SDL_WINDOW_OPENGL | SDL_WINDOW_SHOWN |
@@ -214,16 +311,31 @@ extern "C" int main(int argc, char** argv)
 	}
 #endif
 
+#if defined(TARGET_PANDORA)
+	if (SDL_SetVideoMode(preferences::actual_screen_width(),preferences::actual_screen_height(),16,SDL_FULLSCREEN) == NULL) {
+		std::cerr << "could not set video mode\n";
+		return -1;
+	}
+    EGL_Init();
+#else
 	if (SDL_SetVideoMode(preferences::actual_screen_width(),preferences::actual_screen_height(),0,SDL_OPENGL|(preferences::resizable() ? SDL_RESIZABLE : 0)|(preferences::fullscreen() ? SDL_FULLSCREEN : 0)) == NULL) {
 		std::cerr << "could not set video mode\n";
 		return -1;
 	}
 #endif
 
+#endif
+
 //	srand(time(NULL));
 
 	const stats::manager stats_manager;
-	
+
+	std::cerr
+		<< "\n"
+		<< "OpenGL vendor: " << reinterpret_cast<const char *>(glGetString(GL_VENDOR)) << "\n"
+		<< "OpenGL version: " << reinterpret_cast<const char *>(glGetString(GL_VERSION)) << "\n"
+		<< "OpenGL extensions: " << reinterpret_cast<const char *>(glGetString(GL_EXTENSIONS)) << "\n"
+		<< "\n";
 	glShadeModel(GL_SMOOTH);
 	glEnable(GL_BLEND);
 	glEnable(GL_TEXTURE_2D);
@@ -257,29 +369,34 @@ extern "C" int main(int argc, char** argv)
 	
 	try {
 		wml::schema::init(wml::parse_wml_from_file("data/schema.cfg"));
-		graphical_font::init(wml::parse_wml_from_file("data/fonts.cfg"));
+
+		std::string filename = "data/fonts." + i18n::get_locale() + ".cfg";
+		if (!sys::file_exists(filename))
+			filename = "data/fonts.cfg";
+		graphical_font::init(wml::parse_wml_from_file(filename));
+
 		preloads = wml::parse_wml_from_file("data/preload.cfg");
 		int preload_items = std::distance(preloads->begin_child("preload"), preloads->end_child("preload"));
 		loader.set_number_of_items(preload_items+7); // 7 is the number of items that will be loaded below
 		loader.load(preloads);
-		loader.draw_and_increment("Initializing custom object");
+		loader.draw_and_increment(_("Initializing custom object"));
 		custom_object::init();
-		loader.draw_and_increment("Initializing custom object functions");
+		loader.draw_and_increment(_("Initializing custom object functions"));
 		init_custom_object_functions(wml::parse_wml_from_file("data/functions.cfg"));
-		loader.draw_and_increment("Initializing tiles");
+		loader.draw_and_increment(_("Initializing tiles"));
 		tile_map::init(wml::parse_wml_from_file("data/tiles.cfg",
 		               wml::schema::get("tiles")));
-		loader.draw_and_increment("Initializing GUI");
+		loader.draw_and_increment(_("Initializing GUI"));
 
 		wml::const_node_ptr gui_node = wml::parse_wml_from_file(preferences::load_compiled() ? "data/compiled/gui.cfg" : "data/gui.cfg");
 		gui_section::init(gui_node);
-		loader.draw_and_increment("Initializing GUI");
+		loader.draw_and_increment(_("Initializing GUI"));
 		framed_gui_element::init(gui_node);
 	} catch(const wml::parse_error& e) {
 		return 0;
 	}
 	
-	loader.draw("Loading level");
+	loader.draw(_("Loading level"));
 
 	if(!skip_tests && !test::run_tests()) {
 		return -1;
@@ -322,11 +439,21 @@ extern "C" int main(int argc, char** argv)
 	}
 
 	formula_profiler::manager profiler(profile_output);
+	texture_frame_buffer::init();
 
 	bool quit = false;
+	bool of_initialized = false;
 
 	while(!quit && !show_title_screen(level_cfg)) {
 		boost::intrusive_ptr<level> lvl(load_level(level_cfg));
+		
+#if TARGET_OS_IPHONE || TARGET_IPHONE_SIMULATOR
+		if (!of_initialized)
+		{
+			of_init();
+			of_initialized = true;
+		}
+#endif
 
 		//see if we're loading a multiplayer level, in which case we
 		//connect to the server.
@@ -339,10 +466,13 @@ extern "C" int main(int argc, char** argv)
 			last_draw_position() = screen_position();
 			std::string level_cfg = "waiting-room.cfg";
 			boost::intrusive_ptr<level> wait_lvl(load_level(level_cfg));
+			wait_lvl->finish_loading();
 			wait_lvl->set_multiplayer_slot(0);
 			if(wait_lvl->player()) {
 				wait_lvl->player()->set_current_level(level_cfg);
 			}
+			wait_lvl->set_as_current_level();
+
 			level_runner runner(wait_lvl, level_cfg, orig_level_cfg);
 
 			multiplayer::sync_start_time(*lvl, boost::bind(&level_runner::play_cycle, &runner));
@@ -374,6 +504,10 @@ extern "C" int main(int argc, char** argv)
 	} //end manager scope, make managers destruct before calling SDL_Quit
 
 //	controls::debug_dump_controls();
+#if defined(TARGET_PANDORA)
+    EGL_Destroy();
+#endif
+
 	SDL_Quit();
 	
 	preferences::save_preferences();
