@@ -3,9 +3,13 @@
 #include <string>
 #include <SDL.h>
 
+#include <boost/lexical_cast.hpp>
+
 #include "controls.hpp"
 #include "filesystem.hpp"
 #include "formatter.hpp"
+#include "formula_callable.hpp"
+#include "game_registry.hpp"
 #include "preferences.hpp"
 #include "sound.hpp"
 #include "wml_node.hpp"
@@ -17,7 +21,7 @@
 
 namespace preferences {
 	const std::string& version() {
-		static const std::string Version = "1.0.3";
+		static const std::string Version = "1.1";
 		return Version;
 	}
 
@@ -29,15 +33,21 @@ namespace preferences {
 		bool no_sound_ = false;
 		bool no_music_ = false;
 		bool show_debug_hitboxes_ = false;
+		bool show_iphone_controls_ = false;
 		bool use_pretty_scaling_ = false;
 		bool fullscreen_ = false;
 		bool resizable_ = false;
 		bool debug_ = true;
 		bool reverse_ab_ = false;
+		bool show_fps_ = false;
+		int frame_time_millis_ = 20;
 
 		std::string level_path_ = "data/level/";
 
 		bool send_stats_ = true;
+		bool relay_through_server_ = false;
+		
+		std::string control_scheme_ = "iphone_2d";
 		
 #if TARGET_OS_IPHONE || TARGET_IPHONE_SIMULATOR
 		
@@ -54,8 +64,7 @@ namespace preferences {
 		int actual_screen_height_ = 480;
 
 		bool screen_rotated_ = true;
-
-		bool show_fps_ = false;
+		
 		bool use_joystick_ = false;
 		
 		bool load_compiled_ = true;
@@ -74,12 +83,16 @@ namespace preferences {
 		int actual_screen_height_ = 600;
 		
 		bool screen_rotated_ = false;
-
-		bool show_fps_ = false;
+		
 		bool use_joystick_ = true;
 
 		bool load_compiled_ = false;
 
+#if defined(TARGET_PANDORA)
+        bool use_fbo_ = true;
+        bool use_bequ_ = true;
+#endif
+
 		bool use_16bpp_textures_ = false;
 #endif
 
@@ -106,7 +119,7 @@ namespace preferences {
 
 	int xypos_draw_mask = actual_screen_width_ < virtual_screen_width_ ? ~1 : ~0;
 	bool double_scale() {
-		return xypos_draw_mask&1 != 0;
+		return xypos_draw_mask&1;
 	}
 	bool compiling_tiles = false;
 
@@ -159,7 +172,11 @@ namespace preferences {
 	namespace {
 	void expand_path(std::string& str) {
 		if(!str.empty() && str[0] == '~') {
+#if defined(TARGET_PANDORA)
+			str = std::string(getenv("PWD")) + std::string(str.begin()+1, str.end());
+#else
 			str = std::string(getenv("HOME")) + std::string(str.begin()+1, str.end());
+#endif
 		}
 	}
 	}
@@ -175,6 +192,10 @@ namespace preferences {
 		return show_debug_hitboxes_;
 	}
 	
+	bool show_iphone_controls() {
+		return show_iphone_controls_;
+	}
+	
 	bool use_pretty_scaling() {
 		return use_pretty_scaling_;
 	}
@@ -203,6 +224,16 @@ namespace preferences {
 		reverse_ab_ = value;
 	}
 	
+	const std::string& control_scheme()
+	{
+		return control_scheme_;
+	}
+	
+	void set_control_scheme(const std::string& scheme)
+	{
+		control_scheme_ = scheme;
+	}
+	
 	void set_widescreen()
 	{
 		virtual_screen_width_ = (virtual_screen_height_*16)/9;
@@ -268,6 +299,28 @@ namespace preferences {
 		load_compiled_ = value;
 	}
 	
+#if defined(TARGET_PANDORA)
+	bool use_fbo()
+	{
+		return use_fbo_;
+	}
+
+	bool use_bequ()
+	{
+		return use_bequ_;
+	}
+
+    void set_fbo( bool value )
+    {
+		use_fbo_ = value;
+    }
+
+    void set_bequ( bool value )
+    {
+		use_bequ_ = value;
+    }
+#endif
+
 	bool force_no_npot_textures()
 	{
 		return force_no_npot_textures_;
@@ -287,12 +340,22 @@ namespace preferences {
 	{
 		return show_fps_;
 	}
+	
+	int frame_time_millis()
+	{
+		return frame_time_millis_;
+	}
 
 	bool use_joystick()
 	{
 		return use_joystick_;
 	}
 
+	game_logic::formula_callable* registry()
+	{
+		return &game_registry::instance();
+	}
+
 	void load_preferences()
 	{
 		std::string path = PREFERENCES_PATH;
@@ -317,6 +380,12 @@ namespace preferences {
 		sound::set_music_volume(wml::get_int(node, "music_volume", 1000)/1000.0);
 		sound::set_sound_volume(wml::get_int(node, "sound_volume", 1000)/1000.0);
 
+		const wml::const_node_ptr registry_node = node->get_child("registry");
+		if(registry_node) {
+			game_registry::instance().set_contents(registry_node);
+		}
+
+#if !TARGET_OS_IPHONE && !TARGET_IPHONE_SIMULATOR
 		controls::set_sdlkey(controls::CONTROL_UP, static_cast<SDLKey>(wml::get_int(node, "key_up", SDLK_UP)));
 		controls::set_sdlkey(controls::CONTROL_DOWN, static_cast<SDLKey>(wml::get_int(node, "key_down", SDLK_DOWN)));
 		controls::set_sdlkey(controls::CONTROL_LEFT, static_cast<SDLKey>(wml::get_int(node, "key_left", SDLK_LEFT)));
@@ -324,6 +393,7 @@ namespace preferences {
 		controls::set_sdlkey(controls::CONTROL_ATTACK, static_cast<SDLKey>(wml::get_int(node, "key_attack", SDLK_d)));
 		controls::set_sdlkey(controls::CONTROL_JUMP, static_cast<SDLKey>(wml::get_int(node, "key_jump", SDLK_a)));
 		controls::set_sdlkey(controls::CONTROL_TONGUE, static_cast<SDLKey>(wml::get_int(node, "key_tongue", SDLK_s)));
+#endif
 	}
 
 	void save_preferences()
@@ -343,6 +413,10 @@ namespace preferences {
 		node->set_attr("key_attack", formatter() << controls::get_sdlkey(controls::CONTROL_ATTACK));
 		node->set_attr("key_jump", formatter() << controls::get_sdlkey(controls::CONTROL_JUMP));
 		node->set_attr("key_tongue", formatter() << controls::get_sdlkey(controls::CONTROL_TONGUE));
+
+		wml::node_ptr registry_node(new wml::node("registry"));
+		game_registry::instance().write_contents(registry_node);
+		node->add_child(registry_node);
 		sys::write_file(preferences_path_ + "preferences.cfg", wml::output(node));
 	}
 
@@ -372,6 +446,8 @@ namespace preferences {
 			level_path_ = arg_value + "/";
 		} else if(s == "--show-hitboxes") {
 			show_debug_hitboxes_ = true;
+		} else if(s == "--show-controls") {
+			show_iphone_controls_ = true;
 		} else if(s == "--scale") {
 			set_use_pretty_scaling(true);
 		} else if(s == "--no-sound") {
@@ -384,6 +460,8 @@ namespace preferences {
 			no_music_ = false;
 		} else if(s == "--fullscreen") {
 			fullscreen_ = true;
+		} else if(s == "--windowed") {
+			fullscreen_ = false;
 		} else if(s == "--resizable") {
 			resizable_ = true;
 		} else if(s == "--no-resizable") {
@@ -416,6 +494,7 @@ namespace preferences {
 			recalculate_draw_mask();
 		} else if(s == "--simipad") {
 			sim_iphone_ = true;
+			control_scheme_ = "ipad_2d";
 
 			virtual_screen_width_ = 1024;
 			virtual_screen_height_ = 768;
@@ -436,6 +515,9 @@ namespace preferences {
 			show_fps_ = true;
 		} else if(s == "--no-fps") {
 			show_fps_ = false;
+		} else if(arg_name == "--set-fps" && !arg_value.empty()) {
+			frame_time_millis_ = 1000/boost::lexical_cast<int, std::string>(arg_value);
+			std::cerr << "FPS: " << arg_value << " = " << frame_time_millis_ << "ms/frame\n";
 		} else if(arg_name == "--config-path" && !arg_value.empty()) {
 			set_preferences_path(arg_value);
 		} else if(s == "--no-send-stats") {
@@ -444,6 +526,8 @@ namespace preferences {
 			use_joystick_ = true;
 		} else if(s == "--no-joystick") {
 			use_joystick_ = false;
+		} else if(s == "--relay") {
+			relay_through_server_ = true;
 		} else {
 			return false;
 		}
@@ -462,4 +546,12 @@ namespace preferences {
 	bool send_stats() {
 		return send_stats_;
 	}
+
+	bool relay_through_server() {
+		return relay_through_server_;
+	}
+
+	void set_relay_through_server(bool value) {
+		relay_through_server_ = value;
+	}
 }
