@@ -3,6 +3,10 @@
 
 #include <string>
 
+namespace game_logic {
+class formula_callable;
+}
+
 namespace preferences {
 
 	const std::string& version();
@@ -22,6 +26,7 @@ namespace preferences {
 	const char* auto_save_file_path();
 	void expand_data_paths();
 	bool show_debug_hitboxes();
+	bool show_iphone_controls(); //iphone control hit rects
 	bool use_pretty_scaling();
 	void set_use_pretty_scaling(bool value);
 	bool fullscreen();
@@ -33,6 +38,10 @@ namespace preferences {
 	bool reverse_ab();
 	void set_reverse_ab(bool value);
 	
+	// Control scheme to use on iOS or other touch systems
+	const std::string& control_scheme();
+	void set_control_scheme(const std::string& scheme);
+	
 	void set_widescreen();
 	
 	int virtual_screen_width();
@@ -43,6 +52,8 @@ namespace preferences {
 
 	//whether we are debugging
 	bool debug();
+	
+	int frame_time_millis();
 
 	//whether to show the fps display at the top
 	bool show_fps();
@@ -54,6 +65,13 @@ namespace preferences {
 
 	void set_load_compiled(bool value);
 
+#if defined(TARGET_PANDORA)
+    bool use_fbo();
+    bool use_bequ();
+    void set_fbo( bool value );
+    void set_bequ( bool value );
+#endif
+
 	//this is the mask which we apply to all x,y values before drawing, to
 	//avoid drawing things at "half pixels" when the actual screen dimensions
 	//are lower than the virtual screen dimensions.
@@ -80,6 +98,11 @@ namespace preferences {
 
 	bool send_stats();
 
+	bool relay_through_server();
+	void set_relay_through_server(bool value);
+
+	game_logic::formula_callable* registry();
+
 	void load_preferences();
 	void save_preferences();
 
