@@ -10,8 +10,13 @@
  
  See the COPYING file for more details.
  */
+#ifdef TARGET_PANDORA
+#include <GLES/gl.h>
+#include <GLES/glues.h>
+#else
 #include <GL/gl.h>
 #include <GL/glu.h>
+#endif
 #include <SDL.h>
 
 #include "asserts.hpp"
@@ -28,10 +33,15 @@
 namespace graphics
 {
 
-void set_video_mode(int w, int h)
+bool set_video_mode(int w, int h)
+{
+	return set_video_mode(w,h,0,SDL_OPENGL|(preferences::resizable() ? SDL_RESIZABLE : 0)|(preferences::fullscreen() ? SDL_FULLSCREEN : 0)) != NULL;
+}
+
+SDL_Surface* set_video_mode(int w, int h, int bitsperpixel, int flags)
 {
 	graphics::texture::unbuild_all();
-	SDL_SetVideoMode(w,h,0,SDL_OPENGL|(preferences::resizable() ? SDL_RESIZABLE : 0)|(preferences::fullscreen() ? SDL_FULLSCREEN : 0));
+	SDL_Surface* result = SDL_SetVideoMode(w,h,bitsperpixel,flags);
 	graphics::texture::rebuild_all();
 
 	glShadeModel(GL_SMOOTH);
@@ -41,6 +51,7 @@ void set_video_mode(int w, int h)
 	glEnableClientState(GL_TEXTURE_COORD_ARRAY);
 	
 	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+	return result;
 }
 	
 	/* unavoidable global variable to store global clip
@@ -480,7 +491,7 @@ void queue_blit_texture(const texture& tex, int x, int y, int w, int h,
 	blit_vqueue.push_back(y + h);
 }
 
-void queue_blit_texture(const texture& tex, int x, int y, int w, int h, int rotate,
+void queue_blit_texture(const texture& tex, int x, int y, int w, int h, GLfloat rotate,
 						GLfloat x1, GLfloat y1, GLfloat x2, GLfloat y2)
 {
 	x &= preferences::xypos_draw_mask;
