@@ -1,5 +1,8 @@
+#ifdef TARGET_PANDORA
+#include <GLES/gl.h>
+#else
 #include <GL/gl.h>
-
+#endif
 #include <map>
 #include <sstream>
 #include <stdio.h>
